@@ -109,18 +109,16 @@
 
 (defconst remind-keywords
   (sort
-   (list "ADDOMIT" "AFTER" "AT" "BANNER" "BEFORE"
-         "CAL" "CLEAR-OMIT-CONTEXT" "DEBUG" "DO" "DUMPVARS"
-         "DURATION" "ELSE" "ENDIF" "ERRMSG" "EXIT" "FIRST"
-         "FLUSH" "FOURTH" "FROM" "FSET" "IF" "IFTRIG" "IN"
-         "INCLUDE" "INCLUDECMD" "LAST" "LASTDAY"
-         "LASTWORKDAY" "MAYBE-UNCOMPUTABLE" "MSF"
-         "MSG" "OMIT" "OMITFUNC" "ONCE"
-         "POP-OMIT-CONTEXT" "PRESERVE" "PRIORITY" "PS" "PSFILE"
-         "PUSH-OMIT-CONTEXT" "REM" "RUN" "SATISFY" "SCANFROM"
-         "SCHED" "SECOND" "SET" "SKIP" "SPECIAL"
-         "TAG" "THIRD" "THROUGH" "UNSET" "UNTIL"
-         "WARN")
+   (list "ADDOMIT" "AFTER" "AT" "BAN" "BANNER" "BEFORE" "CAL" "CLEAR"
+         "CLEAR-OMIT-CONTEXT" "DEBUG" "DO" "DUMP" "DUMPVARS" "DURATION" "ELSE"
+         "ENDIF" "ERRMSG" "EXIT" "EXPR" "FIRST" "FLUSH" "FOURTH" "FRENAME" "FROM" "FSET"
+         "FUNSET" "IF" "IFTRIG" "IN" "INC" "INCLUDE" "INCLUDECMD" "LAST"
+         "LASTDAY" "LASTWORKDAY" "MAYBE" "MAYBE-UNCOMPUTABLE" "MSF" "MSG"
+         "NOQUEUE" "OMIT" "OMITFUNC" "ONCE" "POP" "POP-OMIT-CONTEXT" "PRESERVE"
+         "PRIORITY" "PS" "PSFILE" "PUSH" "PUSH-OMIT-CONTEXT" "REM" "RUN"
+         "SATISFY" "SCAN" "SCANFROM" "SCHED" "SECOND" "SET" "SKIP" "SPECIAL"
+         "SYSINCLUDE" "TAG" "THIRD" "THROUGH" "TRANSLATE" "TRANS" "UNSET"
+         "UNTIL" "WARN")
    #'(lambda (a b) (> (length a) (length b)))))
 
 
@@ -131,50 +129,58 @@
 
 (defconst remind-builtin-variables
   (sort
-   (list "$Ago" "$Am" "$And" "$April" "$At" "$August" "$CalcUTC" "$CalMode" "$Daemon" "$DateSep"
-         "$DateTimeSep" "$December" "$DefaultColor" "$DefaultPrio"
-         "$DefaultTDelta" "$DeltaOffset" "$DontFork" "$DontQueue"
-         "$DontTrigAts" "$EndSent" "$EndSentIg" "$February" "$FirstIndent"
-         "$FoldYear" "$FormWidth" "$Friday" "$Fromnow" "$Hour" "$Hplu" "$HushMode" "$IgnoreOnce"
-         "$InfDelta" "$IntMax" "$IntMin" "$Is" "$January" "$July" "$June" "$LatDeg"
-         "$Latitude" "$LatMin" "$LatSec" "$Location" "$LongDeg" "$Longitude"
-         "$LongMin" "$LongSec" "$March" "$MaxSatIter" "$MaxStringLen" "$May"
-         "$MinsFromUTC" "$Minute" "$Monday" "$Mplu" "$NextMode" "$November" "$Now" "$NumQueued"
-         "$NumTrig" "$October" "$On" "$Pm" "$PrefixLineNo" "$PSCal" "$RunOff" "$Saturday"
-         "$September" "$SimpleCal" "$SortByDate" "$SortByPrio" "$SortByTime"
-         "$SubsIndent" "$Sunday" "$SysInclude" "$T" "$Td" "$Thursday" "$TimeSep" "$Tm"
-         "$Today" "$Tomorrow" "$Tuesday" "$Tw" "$Ty" "$U" "$Ud" "$Um" "$UntimedFirst" "$Uw" "$Uy"
-         "$Was" "$Wednesday")
+   (list " $AddBlankLines" "$Ago" "$Am" "$And" "$April" "$At" "$August"
+         "$CalcUTC" "$CalMode" "$CalType" "$Daemon" "$DateSep" "$DateTimeSep"
+         "$December" "$DedupeReminders" "$DefaultColor" "$DefaultDelta"
+         "$DefaultPrio" "$DefaultTDelta" "$DeltaOverride"
+         "$DontFork" "$DontQueue" "$DontTrigAts" "$EndSent" "$EndSentIg"
+         "$ExpressionTimeLimit" "$February" "$FirstIndent" "$FoldYear"
+         "$FormWidth" "$Friday" "$Fromnow" "$Hour" "$Hplu" "$HushMode"
+         "$IgnoreOnce" "$InfDelta" "$IntMax" "$IntMin" "$Is" "$January" "$July"
+         "$June" "$LatDeg" "$Latitude" "$LatMin" "$LatSec" "$Location"
+         "$LongDeg" "$Longitude" "$LongMin" "$LongSec" "$March" "$MaxFullOmits"
+         "$MaxLateMinutes" "$MaxPartialOmits" "$MaxSatIter" "$MaxStringLen"
+         "$May" "$MinsFromUTC" "$Minute" "$Monday" "$Mplu" "$NextMode"
+         "$November" "$Now" "$NumFullOmits" "$NumPartialOmits" "$NumQueued"
+         "$NumTrig" "$October" "$On" "$OnceFile" "$ParseUntriggered" "$Pm"
+         "$PrefixLineNo" "$PSCal" "$RunOff" "$Saturday" "$September"
+         "$SimpleCal" "$SortByDate" "$SortByPrio" "$SortByTime" "$SubsIndent"
+         "$Sunday" "$SuppressImplicitWarnings" "$SuppressLRM" "$SysInclude" "$T" "$Td"
+         "$TerminalBackground" "$Thursday" "$TimeSep" "$Tm" "$Today"
+         "$Tomorrow" "$Tt" "$Tuesday" "$Tw" "$Ty" "$U" "$Ud" "$Um"
+         "$UntimedFirst" "$Use256Colors" "$UseBGVTColors" "$UseTrueColors"
+         "$UseVTColors" "$Uw" "$Uy" "$Was" "$Wednesday")
    #'(lambda (a b) (> (length a) (length b)))))
 
-
 (defconst remind-time-words
   (sort
-   (list "Jan" "January" "Feb" "Mar" "Apr" "Jun" "Jul" "Aug" "Sept" "Sep" "Oct" "Nov" "Dec"
-	 "February" "March" "April" "May" "June" "July" "August" "September" "October"
-	 "November" "December" "Mon" "Monday" "Tue" "Tues" "Tuesday" "Wed" "Wednesday"
-	 "Thu" "Thursday" "Thurs" "Fri" "Friday" "Saturday" "Sat" "Sun" "Sunday")
+   (list "Apr" "April" "Aug" "August" "Dec" "December" "Feb" "February"
+         "Jan" "January" "Jul" "July" "Jun" "June" "Mar" "March" "May"
+         "Nov" "November" "Oct" "October" "Sep" "September" "Fri"
+         "Friday" "Mon" "Monday" "Sat" "Saturday" "Sun" "Sunday" "Thu"
+         "Thursday" "Tue" "Tuesday" "Wed" "Wednesday")
    #'(lambda (a b) (> (length a) (length b)))))
 
-
 (defconst remind-builtin-functions
   (sort
-   (list "abs" "access" "adawn" "adusk" "ampm" "args" "asc" "baseyr" "char"
-         "choose" "coerce" "current" "date" "datepart" "datetime" "dawn" "day"
-         "daysinmon" "defined" "dosubst" "dusk" "easterdate" "evaltrig"
-         "filedate" "filedatetime" "filedir" "filename" "getenv" "hebdate"
-         "hebday" "hebmon" "hebyear" "hour" "iif" "index" "isany" "isdst"
-         "isleap" "isomitted" "language" "lower" "max" "min" "minsfromutc"
+   (list "_" "abs" "access" "adawn" "adusk" "ampm" "ansicolor" "args" "asc"
+         "baseyr" "char" "choose" "coerce" "columns" "current" "date"
+         "datepart" "datetime" "dawn" "day" "daysinmon" "defined" "dosubst"
+         "dusk" "easterdate" "evaltrig" "filedate" "filedatetime" "filedir"
+         "filename" "getenv" "hebdate" "hebday" "hebmon" "hebyear" "hour"
+         "htmlescape" "htmlstriptags" "iif" "index" "isany" "isdst" "isleap"
+         "isomitted" "language" "localtoutc" "lower" "max" "min" "minsfromutc"
          "minute" "mon" "monnum" "moondate" "moondatetime" "moonphase"
-         "moontime" "ndawn" "ndusk" "nonomitted" "now" "ord" "ostype" "pad" "plural"
-         "psmoon" "psshade" "realcurrent" "realnow" "realtoday" "sgn" "shell"
-         "shellescape" "slide" "strlen" "substr" "sunrise" "sunset" "time"
-         "timepart" "today" "trig" "trigback" "trigdate" "trigdatetime"
-         "trigdelta" "trigduration" "trigeventduration" "trigeventstart"
-         "trigfrom" "trigger" "trigpriority" "trigrep" "trigscanfrom"
-         "trigtime" "trigtimedelta" "trigtimerep" "triguntil" "trigvalid"
-         "typeof" "tzconvert" "upper" "value" "version" "weekno" "wkday"
-         "wkdaynum" "year")
+         "moontime" "multitrig" "ndawn" "ndusk" "nonomitted" "now" "ord"
+         "orthodoxeaster" "ostype" "pad" "plural" "psmoon" "psshade"
+         "realcurrent" "realnow" "realtoday" "rows" "sgn" "shell" "shellescape"
+         "slide" "soleq" "stdout" "strlen" "substr" "sunrise" "sunset" "time"
+         "timepart" "timezone" "today" "trig" "trigback" "trigdate"
+         "trigdatetime" "trigdelta" "trigduration" "trigeventduration"
+         "trigeventstart" "trigfrom" "trigger" "trigpriority" "trigrep"
+         "trigscanfrom" "trigtags" "trigtime" "trigtimedelta" "trigtimerep"
+         "triguntil" "trigvalid" "typeof" "tzconvert" "upper" "utctolocal"
+         "value" "version" "weekno" "wkday" "wkdaynum" "year")
    #'(lambda (a b) (> (length a) (length b)))))
 
 ;;; faces
@@ -183,21 +189,21 @@
 (defvar remind-conf-command-face 'remind-conf-command-face
   "Remind commands.")
 (defface remind-conf-command-face
-  '((t :foreground "SeaGreen4" :bold t))
+  '((t :foreground "#FF8080"))
   "Font Lock mode face used to highlight commands."
   :group 'remind-conf)
 
 (defvar remind-conf-keyword-face 'remind-conf-keyword-face
   "Remind keywords.")
 (defface remind-conf-keyword-face
-  '((t :foreground "blue violet"))
+  '((t :foreground "#FFFF80"))
   "Font Lock mode face used to highlight keywords."
   :group 'remind-conf)
 
 (defvar remind-conf-substitutes-face 'remind-conf-substitutes-face
   "Remind substitutes.")
 (defface remind-conf-substitutes-face
-  '((t :foreground "blue2"))
+  '((t :foreground "#8080FF"))
   "Font Lock mode face used to highlight substitutes."
   :group 'remind-conf)
 
@@ -211,7 +217,7 @@
 (defvar remind-conf-variable-face 'remind-conf-variable-face
   "Remind variable.")
 (defface remind-conf-variable-face
-  '((t :foreground "DeepPink2" :bold t))
+  '((t :foreground "#FF8080" :bold t))
   "Font Lock mode face used to highlight commands."
   :group 'remind-conf)
 
@@ -232,14 +238,14 @@
 (defvar remind-comment-face 'remind-comment-face
   "Remind comments.")
 (defface remind-comment-face
-  '((t :foreground "brown"))
+  '((t :foreground "#FF7F24"))
   "Font-lock face for highlighting comments."
   :group 'remind-conf)
 
 (defvar remind-string-face 'remind-string-face
   "Remind strings.")
 (defface remind-string-face
-  '((t :foreground "tomato"))
+  '((t :foreground "#FF7FE0"))
   "Font lock mode face used to highlight strings."
   :group 'remind-conf)
 
@@ -290,12 +296,12 @@
 
 (defconst remind-conf-font-lock-keywords-1
   (list
-   '("^[\;\#]\\s-+.*$" . remind-comment-face)
+   '("^\s*[\;\#].*$" . remind-comment-face)
+   '("\"[^\"]*\"" . remind-string-face)
    '(remind-keywords-matcher . remind-conf-keyword-face)
    '("%[\"_]" . font-lock-warning-face)
-   '("\\(%[a-mops-w]\\)" . remind-conf-substitutes-face)
-   '("\"[^\"]*\"" . remind-string-face))
-  "Minimal font-locking for `remind-conf-mode'.")
+   '("\\(%[a-z]\\)" . remind-conf-substitutes-face)
+  "Minimal font-locking for `remind-conf-mode'."))
 
 (defconst remind-conf-font-lock-keywords-2
   (append remind-conf-font-lock-keywords-1
