@@ -7,7 +7,7 @@
 /*  files.                                                     */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2024 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2025 by Dianne Skoll                    */
 /*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
@@ -102,6 +102,13 @@ static int CheckSafetyAux (struct stat *statbuf);
 static int PopFile (void);
 static int IncludeCmd(char const *);
 
+static void
+got_a_fresh_line(void)
+{
+    FreshLine = 1;
+    WarnedAboutImplicit = 0;
+}
+
 void set_cloexec(FILE *fp)
 {
     int flags;
@@ -122,15 +129,15 @@ static void OpenPurgeFile(char const *fname, char const *mode)
     DynamicBuffer fname_buf;
 
     if (PurgeFP != NULL && PurgeFP != stdout) {
-	fclose(PurgeFP);
+        fclose(PurgeFP);
     }
     PurgeFP = NULL;
 
     /* Do not open a purge file if we're below purge
        include depth */
     if (IStackPtr-2 >= PurgeIncludeDepth) {
-	PurgeFP = NULL;
-	return;
+        PurgeFP = NULL;
+        return;
     }
 
     DBufInit(&fname_buf);
@@ -138,7 +145,7 @@ static void OpenPurgeFile(char const *fname, char const *mode)
     if (DBufPuts(&fname_buf, ".purged") != OK) return;
     PurgeFP = fopen(DBufValue(&fname_buf), mode);
     if (!PurgeFP) {
-	fprintf(ErrFp, "Cannot open `%s' for writing: %s\n", DBufValue(&fname_buf), strerror(errno));
+        fprintf(ErrFp, "Cannot open `%s' for writing: %s\n", DBufValue(&fname_buf), strerror(errno));
     }
     set_cloexec(PurgeFP);
     DBufFree(&fname_buf);
@@ -146,17 +153,17 @@ static void OpenPurgeFile(char const *fname, char const *mode)
 
 static void FreeChainItem(FilenameChain *chain)
 {
-	if (chain->filename) free((void *) chain->filename);
-	free(chain);
+        if (chain->filename) free((void *) chain->filename);
+        free(chain);
 }
 
 static void FreeChain(FilenameChain *chain)
 {
     FilenameChain *next;
     while(chain) {
-	next = chain->next;
-	FreeChainItem(chain);
-	chain = next;
+        next = chain->next;
+        FreeChainItem(chain);
+        chain = next;
     }
 }
 
@@ -173,19 +180,19 @@ int ReadLine(void)
 
 /* If we're at the end of a file, pop */
     while (!CLine && !fp) {
-	r = PopFile();
-	if (r) return r;
+        r = PopFile();
+        if (r) return r;
     }
 
 /* If it's cached, read line from the cache */
     if (CLine) {
-	CurLine = CLine->text;
-	LineNo = CLine->LineNo;
-	CLine = CLine->next;
-	FreshLine = 1;
+        CurLine = CLine->text;
+        LineNo = CLine->LineNo;
+        CLine = CLine->next;
+        got_a_fresh_line();
         clear_callstack();
-	if (DebugFlag & DB_ECHO_LINE) OutputLine(ErrFp);
-	return OK;
+        if (DebugFlag & DB_ECHO_LINE) OutputLine(ErrFp);
+        return OK;
     }
 
 /* Not cached.  Read from the file. */
@@ -211,82 +218,82 @@ static int ReadLineFromFile(int use_pclose)
     DBufFree(&LineBuffer);
 
     while(fp) {
-	if (DBufGets(&buf, fp) != OK) {
-	    DBufFree(&LineBuffer);
-	    return E_NO_MEM;
-	}
-	LineNo++;
-	if (ferror(fp)) {
-	    DBufFree(&buf);
-	    DBufFree(&LineBuffer);
-	    return E_IO_ERR;
-	}
-	if (feof(fp)) {
+        if (DBufGets(&buf, fp) != OK) {
+            DBufFree(&LineBuffer);
+            return E_NO_MEM;
+        }
+        LineNo++;
+        if (ferror(fp)) {
+            DBufFree(&buf);
+            DBufFree(&LineBuffer);
+            return E_IO_ERR;
+        }
+        if (feof(fp)) {
             if (use_pclose) {
                 PCLOSE(fp);
             } else {
                 FCLOSE(fp);
             }
-	    if ((DBufLen(&buf) == 0) &&
-		(DBufLen(&LineBuffer) == 0) && PurgeMode) {
-		if (PurgeFP != NULL && PurgeFP != stdout) fclose(PurgeFP);
-		PurgeFP = NULL;
-	    }
-	}
-	l = DBufLen(&buf);
-	if (l && (DBufValue(&buf)[l-1] == '\\')) {
-	    if (PurgeMode) {
-		if (DBufPuts(&LineBuffer, DBufValue(&buf)) != OK) {
-		    DBufFree(&buf);
-		    DBufFree(&LineBuffer);
-		    return E_NO_MEM;
-		}
-		if (DBufPutc(&LineBuffer, '\n') != OK) {
-		    DBufFree(&buf);
-		    DBufFree(&LineBuffer);
-		    return E_NO_MEM;
-		}
-	    } else {
-		DBufValue(&buf)[l-1] = '\n';
-		if (DBufPuts(&LineBuffer, DBufValue(&buf)) != OK) {
-		    DBufFree(&buf);
-		    DBufFree(&LineBuffer);
-		    return E_NO_MEM;
-		}
-	    }
-	    continue;
-	}
-	if (DBufPuts(&LineBuffer, DBufValue(&buf)) != OK) {
-	    DBufFree(&buf);
-	    DBufFree(&LineBuffer);
-	    return E_NO_MEM;
-	}
-	DBufFree(&buf);
+            if ((DBufLen(&buf) == 0) &&
+                (DBufLen(&LineBuffer) == 0) && PurgeMode) {
+                if (PurgeFP != NULL && PurgeFP != stdout) fclose(PurgeFP);
+                PurgeFP = NULL;
+            }
+        }
+        l = DBufLen(&buf);
+        if (l && (DBufValue(&buf)[l-1] == '\\')) {
+            if (PurgeMode) {
+                if (DBufPuts(&LineBuffer, DBufValue(&buf)) != OK) {
+                    DBufFree(&buf);
+                    DBufFree(&LineBuffer);
+                    return E_NO_MEM;
+                }
+                if (DBufPutc(&LineBuffer, '\n') != OK) {
+                    DBufFree(&buf);
+                    DBufFree(&LineBuffer);
+                    return E_NO_MEM;
+                }
+            } else {
+                DBufValue(&buf)[l-1] = '\n';
+                if (DBufPuts(&LineBuffer, DBufValue(&buf)) != OK) {
+                    DBufFree(&buf);
+                    DBufFree(&LineBuffer);
+                    return E_NO_MEM;
+                }
+            }
+            continue;
+        }
+        if (DBufPuts(&LineBuffer, DBufValue(&buf)) != OK) {
+            DBufFree(&buf);
+            DBufFree(&LineBuffer);
+            return E_NO_MEM;
+        }
+        DBufFree(&buf);
 
-	/* If the line is: __EOF__ treat it as end-of-file */
-	CurLine = DBufValue(&LineBuffer);
-	if (!strcmp(CurLine, "__EOF__")) {
-	    if (PurgeMode && PurgeFP) {
-		PurgeEchoLine("%s\n", "__EOF__");
-		while ((n = fread(copy_buffer, 1, sizeof(copy_buffer), fp)) != 0) {
-		    fwrite(copy_buffer, 1, n, PurgeFP);
-		}
-		if (PurgeFP != stdout) fclose(PurgeFP);
-		PurgeFP = NULL;
-	    }
+        /* If the line is: __EOF__ treat it as end-of-file */
+        CurLine = DBufValue(&LineBuffer);
+        if (!strcmp(CurLine, "__EOF__")) {
+            if (PurgeMode && PurgeFP) {
+                PurgeEchoLine("%s\n", "__EOF__");
+                while ((n = fread(copy_buffer, 1, sizeof(copy_buffer), fp)) != 0) {
+                    fwrite(copy_buffer, 1, n, PurgeFP);
+                }
+                if (PurgeFP != stdout) fclose(PurgeFP);
+                PurgeFP = NULL;
+            }
             if (use_pclose) {
                 PCLOSE(fp);
             } else {
                 FCLOSE(fp);
             }
-	    DBufFree(&LineBuffer);
-	    CurLine = DBufValue(&LineBuffer);
-	}
+            DBufFree(&LineBuffer);
+            CurLine = DBufValue(&LineBuffer);
+        }
 
-	FreshLine = 1;
+        got_a_fresh_line();
         clear_callstack();
-	if (DebugFlag & DB_ECHO_LINE) OutputLine(ErrFp);
-	return OK;
+        if (DebugFlag & DB_ECHO_LINE) OutputLine(ErrFp);
+        return OK;
     }
     CurLine = DBufValue(&LineBuffer);
     return OK;
@@ -307,71 +314,71 @@ int OpenFile(char const *fname)
     int r;
 
     if (PurgeMode) {
-	if (PurgeFP != NULL && PurgeFP != stdout) {
-	    fclose(PurgeFP);
-	}
-	PurgeFP = NULL;
+        if (PurgeFP != NULL && PurgeFP != stdout) {
+            fclose(PurgeFP);
+        }
+        PurgeFP = NULL;
     }
 
 /* If it's in the cache, get it from there. */
 
     while (h) {
-	if (!strcmp(fname, h->filename)) {
-	    if (DebugFlag & DB_TRACE_FILES) {
-		fprintf(ErrFp, "Reading `%s': Found in cache\n", fname);
-	    }
-	    CLine = h->cache;
-	    STRSET(FileName, fname);
-	    LineNo = 0;
-	    if (!h->ownedByMe) {
-		RunDisabled |= RUN_NOTOWNER;
-	    } else {
-		RunDisabled &= ~RUN_NOTOWNER;
+        if (!strcmp(fname, h->filename)) {
+            if (DebugFlag & DB_TRACE_FILES) {
+                fprintf(ErrFp, "Reading `%s': Found in cache\n", fname);
             }
-	    if (FileName) return OK; else return E_NO_MEM;
-	}
-	h = h->next;
+            CLine = h->cache;
+            STRSET(FileName, fname);
+            LineNo = 0;
+            if (!h->ownedByMe) {
+                RunDisabled |= RUN_NOTOWNER;
+            } else {
+                RunDisabled &= ~RUN_NOTOWNER;
+            }
+            if (FileName) return OK; else return E_NO_MEM;
+        }
+        h = h->next;
     }
 
 /* If it's a dash, then it's stdin */
     if (!strcmp(fname, "-")) {
-	fp = stdin;
+        fp = stdin;
         RunDisabled &= ~RUN_NOTOWNER;
-	if (PurgeMode) {
-	    PurgeFP = stdout;
-	}
-	if (DebugFlag & DB_TRACE_FILES) {
-	    fprintf(ErrFp, "Reading `-': Reading stdin\n");
-	}
+        if (PurgeMode) {
+            PurgeFP = stdout;
+        }
+        if (DebugFlag & DB_TRACE_FILES) {
+            fprintf(ErrFp, "Reading `-': Reading stdin\n");
+        }
     } else {
-	fp = fopen(fname, "r");
+        fp = fopen(fname, "r");
         set_cloexec(fp);
-	if (DebugFlag & DB_TRACE_FILES) {
-	    fprintf(ErrFp, "Reading `%s': Opening file on disk\n", fname);
-	}
-	if (PurgeMode) {
-	    OpenPurgeFile(fname, "w");
-	}
+        if (DebugFlag & DB_TRACE_FILES) {
+            fprintf(ErrFp, "Reading `%s': Opening file on disk\n", fname);
+        }
+        if (PurgeMode) {
+            OpenPurgeFile(fname, "w");
+        }
     }
     if (!fp || !CheckSafety()) return E_CANT_OPEN;
     CLine = NULL;
     if (ShouldCache) {
-	LineNo = 0;
-	r = CacheFile(fname, 0);
-	if (r == OK) {
-	    fp = NULL;
-	    CLine = CachedFiles->cache;
-	} else {
-	    if (strcmp(fname, "-")) {
-		fp = fopen(fname, "r");
-		if (!fp || !CheckSafety()) return E_CANT_OPEN;
+        LineNo = 0;
+        r = CacheFile(fname, 0);
+        if (r == OK) {
+            fp = NULL;
+            CLine = CachedFiles->cache;
+        } else {
+            if (strcmp(fname, "-")) {
+                fp = fopen(fname, "r");
+                if (!fp || !CheckSafety()) return E_CANT_OPEN;
                 set_cloexec(fp);
-		if (PurgeMode) OpenPurgeFile(fname, "w");
-	    } else {
-		fp = stdin;
-		if (PurgeMode) PurgeFP = stdout;
-	    }
-	}
+                if (PurgeMode) OpenPurgeFile(fname, "w");
+            } else {
+                fp = stdin;
+                if (PurgeMode) PurgeFP = stdout;
+            }
+        }
     }
     STRSET(FileName, fname);
     LineNo = 0;
@@ -394,101 +401,101 @@ static int CacheFile(char const *fname, int use_pclose)
     char const *s;
 
     if (DebugFlag & DB_TRACE_FILES) {
-	fprintf(ErrFp, "Caching file `%s' in memory\n", fname);
+        fprintf(ErrFp, "Caching file `%s' in memory\n", fname);
     }
     cl = NULL;
 /* Create a file header */
     cf = NEW(CachedFile);
     if (!cf) {
-	ShouldCache = 0;
+        ShouldCache = 0;
         if (use_pclose) {
             PCLOSE(fp);
         } else {
             FCLOSE(fp);
         }
-	return E_NO_MEM;
+        return E_NO_MEM;
     }
     cf->cache = NULL;
     cf->filename = StrDup(fname);
     if (!cf->filename) {
-	ShouldCache = 0;
+        ShouldCache = 0;
         if (use_pclose) {
             PCLOSE(fp);
         } else {
             FCLOSE(fp);
         }
-	free(cf);
-	return E_NO_MEM;
+        free(cf);
+        return E_NO_MEM;
     }
 
     if (RunDisabled & RUN_NOTOWNER) {
-	cf->ownedByMe = 0;
+        cf->ownedByMe = 0;
     } else {
-	cf->ownedByMe = 1;
+        cf->ownedByMe = 1;
     }
 
 /* Read the file */
     while(fp) {
-	r = ReadLineFromFile(use_pclose);
-	if (r) {
-	    DestroyCache(cf);
-	    ShouldCache = 0;
+        r = ReadLineFromFile(use_pclose);
+        if (r) {
+            DestroyCache(cf);
+            ShouldCache = 0;
             if (use_pclose) {
                 PCLOSE(fp);
             } else {
                 FCLOSE(fp);
             }
-	    return r;
-	}
+            return r;
+        }
 /* Skip blank chars */
-	s = DBufValue(&LineBuffer);
-	while (isempty(*s)) s++;
-	if (*s && *s!=';' && *s!='#') {
+        s = DBufValue(&LineBuffer);
+        while (isempty(*s)) s++;
+        if (*s && *s!=';' && *s!='#') {
 /* Add the line to the cache */
-	    if (!cl) {
-		cf->cache = NEW(CachedLine);
-		if (!cf->cache) {
-		    DBufFree(&LineBuffer);
-		    DestroyCache(cf);
-		    ShouldCache = 0;
+            if (!cl) {
+                cf->cache = NEW(CachedLine);
+                if (!cf->cache) {
+                    DBufFree(&LineBuffer);
+                    DestroyCache(cf);
+                    ShouldCache = 0;
                     if (use_pclose) {
                         PCLOSE(fp);
                     } else {
                         FCLOSE(fp);
                     }
-		    return E_NO_MEM;
-		}
-		cl = cf->cache;
-	    } else {
-		cl->next = NEW(CachedLine);
-		if (!cl->next) {
-		    DBufFree(&LineBuffer);
-		    DestroyCache(cf);
-		    ShouldCache = 0;
+                    return E_NO_MEM;
+                }
+                cl = cf->cache;
+            } else {
+                cl->next = NEW(CachedLine);
+                if (!cl->next) {
+                    DBufFree(&LineBuffer);
+                    DestroyCache(cf);
+                    ShouldCache = 0;
                     if (use_pclose) {
                         PCLOSE(fp);
                     } else {
                         FCLOSE(fp);
                     }
-		    return E_NO_MEM;
-		}
-		cl = cl->next;
-	    }
-	    cl->next = NULL;
-	    cl->LineNo = LineNo;
-	    cl->text = StrDup(s);
-	    DBufFree(&LineBuffer);
-	    if (!cl->text) {
-		DestroyCache(cf);
-		ShouldCache = 0;
+                    return E_NO_MEM;
+                }
+                cl = cl->next;
+            }
+            cl->next = NULL;
+            cl->LineNo = LineNo;
+            cl->text = StrDup(s);
+            DBufFree(&LineBuffer);
+            if (!cl->text) {
+                DestroyCache(cf);
+                ShouldCache = 0;
                 if (use_pclose) {
                     PCLOSE(fp);
                 } else {
                     FCLOSE(fp);
                 }
-		return E_NO_MEM;
-	    }
-	}
+                return E_NO_MEM;
+            }
+        }
     }
 
 /* Put the cached file at the head of the queue */
@@ -507,13 +514,13 @@ static int CacheFile(char const *fname, int use_pclose)
 static int NextChainedFile(IncludeStruct *i)
 {
     while(i->chain) {
-	FilenameChain *cur = i->chain;
-	i->chain = i->chain->next;
-	if (OpenFile(cur->filename) == OK) {
-	    return OK;
-	} else {
-	    Eprint("%s: %s", ErrMsg[E_CANT_OPEN], cur->filename);
-	}
+        FilenameChain *cur = i->chain;
+        i->chain = i->chain->next;
+        if (OpenFile(cur->filename) == OK) {
+            return OK;
+        } else {
+            Eprint("%s: %s", GetErr(E_CANT_OPEN), cur->filename);
+        }
     }
     return E_EOF;
 }
@@ -530,7 +537,7 @@ static int PopFile(void)
     int j;
 
     if (!Hush && NumIfs) {
-        Eprint("%s", ErrMsg[E_MISS_ENDIF]);
+        Eprint("%s", GetErr(E_MISS_ENDIF));
         for (j=NumIfs-1; j >=0; j--) {
             fprintf(ErrFp, "%s(%d): IF without ENDIF\n", FileName, IfLinenos[j]);
         }
@@ -539,15 +546,15 @@ static int PopFile(void)
     i = &IStack[IStackPtr-1];
 
     if (i->chain) {
-	int oldRunDisabled = RunDisabled;
-	if (NextChainedFile(i) == OK) {
-	    return OK;
-	}
-	RunDisabled = oldRunDisabled;
+        int oldRunDisabled = RunDisabled;
+        if (NextChainedFile(i) == OK) {
+            return OK;
+        }
+        RunDisabled = oldRunDisabled;
     }
 
     if (IStackPtr <= 1) {
-	return E_EOF;
+        return E_EOF;
     }
 
     IStackPtr--;
@@ -560,23 +567,23 @@ static int PopFile(void)
     fp = NULL;
     STRSET(FileName, i->filename);
     if (!i->ownedByMe) {
-	RunDisabled |= RUN_NOTOWNER;
+        RunDisabled |= RUN_NOTOWNER;
     } else {
-	RunDisabled &= ~RUN_NOTOWNER;
+        RunDisabled &= ~RUN_NOTOWNER;
     }
     if (!CLine && (i->offset != -1L || !strcmp(i->filename, "-"))) {
-	/* We must open the file, then seek to specified position */
-	if (strcmp(i->filename, "-")) {
-	    fp = fopen(i->filename, "r");
-	    if (!fp || !CheckSafety()) return E_CANT_OPEN;
+        /* We must open the file, then seek to specified position */
+        if (strcmp(i->filename, "-")) {
+            fp = fopen(i->filename, "r");
+            if (!fp || !CheckSafety()) return E_CANT_OPEN;
             set_cloexec(fp);
-	    if (PurgeMode) OpenPurgeFile(i->filename, "a");
-	} else {
-	    fp = stdin;
-	    if (PurgeMode) PurgeFP = stdout;
-	}
-	if (fp != stdin)
-	    (void) fseek(fp, i->offset, 0);  /* Trust that it works... */
+            if (PurgeMode) OpenPurgeFile(i->filename, "a");
+        } else {
+            fp = stdin;
+            if (PurgeMode) PurgeFP = stdout;
+        }
+        if (fp != stdin)
+            (void) fseek(fp, i->offset, 0);  /* Trust that it works... */
     }
     free((char *) i->filename);
     return OK;
@@ -601,16 +608,25 @@ int DoInclude(ParsePtr p, enum TokTypes tok)
     DBufInit(&buf);
     DBufInit(&fullname);
     DBufInit(&path);
-    if ( (r=ParseToken(p, &buf)) ) return r;
+    if ( (r=ParseTokenOrQuotedString(p, &buf)) ) return r;
     e = VerifyEoln(p);
-    if (e) Eprint("%s", ErrMsg[e]);
+    if (e) Eprint("%s", GetErr(e));
 
-    if (tok == T_IncludeR && *(DBufValue(&buf)) != '/') {
+    if ((tok == T_IncludeR || tok == T_IncludeSys) &&
+        *(DBufValue(&buf)) != '/') {
         /* Relative include: Include relative to dir
            containing current file */
-        if (DBufPuts(&path, FileName) != OK) {
-            r = E_NO_MEM;
-            goto bailout;
+        if (tok == T_IncludeR) {
+            if (DBufPuts(&path, FileName) != OK) {
+                r = E_NO_MEM;
+                goto bailout;
+            }
+        } else {
+            if (DBufPuts(&path, SysDir) != OK ||
+                DBufPutc(&path, '/') != OK) {
+                r = E_NO_MEM;
+                goto bailout;
+            }
         }
         if (DBufLen(&path) == 0) {
             s = DBufValue(&buf);
@@ -643,9 +659,6 @@ int DoInclude(ParsePtr p, enum TokTypes tok)
         goto bailout;
     }
 
-    NumIfs = 0;
-    IfFlags = 0;
-
   bailout:
     DBufFree(&buf);
     DBufFree(&path);
@@ -675,9 +688,9 @@ int DoIncludeCmd(ParsePtr p)
     while(1) {
         ch = ParseChar(p, &r, 0);
         if (r) {
-	    DBufFree(&buf);
-	    return r;
-	}
+            DBufFree(&buf);
+            return r;
+        }
         if (!ch) {
             break;
         }
@@ -690,10 +703,10 @@ int DoIncludeCmd(ParsePtr p)
             ch = ' ';
         }
         append_buf[0] = (char) ch;
-	if (DBufPuts(&buf, append_buf) != OK) {
-	    DBufFree(&buf);
-	    return E_NO_MEM;
-	}
+        if (DBufPuts(&buf, append_buf) != OK) {
+            DBufFree(&buf);
+            return E_NO_MEM;
+        }
     }
 
     if (RunDisabled) {
@@ -702,12 +715,10 @@ int DoIncludeCmd(ParsePtr p)
     }
 
     if ( (r=IncludeCmd(DBufValue(&buf))) ) {
-	DBufFree(&buf);
-	return r;
+        DBufFree(&buf);
+        return r;
     }
     DBufFree(&buf);
-    NumIfs = 0;
-    IfFlags = 0;
     return OK;
 }
 
@@ -719,6 +730,8 @@ static int SetupGlobChain(char const *dirname, IncludeStruct *i)
     size_t l;
     int r;
     glob_t glob_buf;
+    struct stat sb;
+
     DirectoryFilenameChain *dc = CachedDirectoryChains;
 
     i->chain = NULL;
@@ -730,55 +743,55 @@ static int SetupGlobChain(char const *dirname, IncludeStruct *i)
     /* Strip trailing slashes off directory */
     l = strlen(dir);
     while(l) {
-	if (*(dir+l-1) == '/') {
-	    l--;
-	    *(dir+l) = 0;
-	} else {
-	    break;
-	}
+        if (*(dir+l-1) == '/') {
+            l--;
+            *(dir+l) = 0;
+        } else {
+            break;
+        }
     }
 
     /* Repair root directory :-) */
     if (!l) {
-	*dir = '/';
+        *dir = '/';
     }
 
     /* Check the cache */
     while(dc) {
-	if (!strcmp(dc->dirname, dir)) {
-	    if (DebugFlag & DB_TRACE_FILES) {
-		fprintf(ErrFp, "Found cached directory listing for `%s'\n",
-			dir);
-	    }
-	    free(dir);
-	    i->chain = dc->chain;
-	    return OK;
-	}
-	dc = dc->next;
+        if (!strcmp(dc->dirname, dir)) {
+            if (DebugFlag & DB_TRACE_FILES) {
+                fprintf(ErrFp, "Found cached directory listing for `%s'\n",
+                        dir);
+            }
+            free(dir);
+            i->chain = dc->chain;
+            return OK;
+        }
+        dc = dc->next;
     }
 
     if (DebugFlag & DB_TRACE_FILES) {
-	fprintf(ErrFp, "Scanning directory `%s' for *.rem files\n", dir);
+        fprintf(ErrFp, "Scanning directory `%s' for *.rem files\n", dir);
     }
 
     if (ShouldCache) {
-	dc = malloc(sizeof(DirectoryFilenameChain));
-	if (dc) {
-	    dc->dirname = StrDup(dir);
-	    if (!dc->dirname) {
-		free(dc);
-		dc = NULL;
-	    }
-	}
-	if (dc) {
-	    if (DebugFlag & DB_TRACE_FILES) {
-		fprintf(ErrFp, "Caching directory `%s' listing\n", dir);
-	    }
+        dc = malloc(sizeof(DirectoryFilenameChain));
+        if (dc) {
+            dc->dirname = StrDup(dir);
+            if (!dc->dirname) {
+                free(dc);
+                dc = NULL;
+            }
+        }
+        if (dc) {
+            if (DebugFlag & DB_TRACE_FILES) {
+                fprintf(ErrFp, "Caching directory `%s' listing\n", dir);
+            }
 
-	    dc->chain = NULL;
-	    dc->next = CachedDirectoryChains;
-	    CachedDirectoryChains = dc;
-	}
+            dc->chain = NULL;
+            dc->next = CachedDirectoryChains;
+            CachedDirectoryChains = dc;
+        }
     }
 
     DBufInit(&pattern);
@@ -790,40 +803,48 @@ static int SetupGlobChain(char const *dirname, IncludeStruct *i)
     DBufFree(&pattern);
 
     if (r == GLOB_NOMATCH) {
-	globfree(&glob_buf);
-	return OK;
+        globfree(&glob_buf);
+        return OK;
     }
 
     if (r != 0) {
-	globfree(&glob_buf);
-	return -1;
+        globfree(&glob_buf);
+        return -1;
     }
 
     /* Add the files to the chain backwards to preserve sort order */
     for (r=glob_buf.gl_pathc-1; r>=0; r--) {
-	FilenameChain *ch = malloc(sizeof(FilenameChain));
-	if (!ch) {
-	    globfree(&glob_buf);
-	    FreeChain(i->chain);
-	    i->chain = NULL;
-	    return E_NO_MEM;
-	}
+        if (stat(glob_buf.gl_pathv[r], &sb) < 0) {
+            /* Couldn't stat the file... fuggedaboutit */
+            continue;
+        }
 
-	/* TODO: stat the file and only add if it's a plain file and
-	   readable by us */
-	ch->filename = StrDup(glob_buf.gl_pathv[r]);
-	if (!ch->filename) {
-	    globfree(&glob_buf);
-	    FreeChain(i->chain);
-	    i->chain = NULL;
-	    free(ch);
-	    return E_NO_MEM;
-	}
-	ch->next = i->chain;
-	i->chain = ch;
+        /* Don't add directories */
+        if (S_ISDIR(sb.st_mode)) {
+            continue;
+        }
+
+        FilenameChain *ch = malloc(sizeof(FilenameChain));
+        if (!ch) {
+            globfree(&glob_buf);
+            FreeChain(i->chain);
+            i->chain = NULL;
+            return E_NO_MEM;
+        }
+
+        ch->filename = StrDup(glob_buf.gl_pathv[r]);
+        if (!ch->filename) {
+            globfree(&glob_buf);
+            FreeChain(i->chain);
+            i->chain = NULL;
+            free(ch);
+            return E_NO_MEM;
+        }
+        ch->next = i->chain;
+        i->chain = ch;
     }
     if (dc) {
-	dc->chain = i->chain;
+        dc->chain = i->chain;
     }
 
     globfree(&glob_buf);
@@ -849,31 +870,31 @@ static int IncludeCmd(char const *cmd)
     char const *fname;
     int old_flag;
 
-    FreshLine = 1;
+    got_a_fresh_line();
     clear_callstack();
-    if (IStackPtr+1 >= INCLUDE_NEST) return E_NESTED_INCLUDE;
+    if (IStackPtr >= INCLUDE_NEST) return E_NESTED_INCLUDE;
     i = &IStack[IStackPtr];
 
     /* Use "cmd|" as the filename */
     DBufInit(&buf);
     if (DBufPuts(&buf, cmd) != OK) {
-	DBufFree(&buf);
-	return E_NO_MEM;
+        DBufFree(&buf);
+        return E_NO_MEM;
     }
     if (DBufPuts(&buf, "|") != OK) {
-	DBufFree(&buf);
-	return E_NO_MEM;
+        DBufFree(&buf);
+        return E_NO_MEM;
     }
     fname = DBufValue(&buf);
 
     if (FileName) {
-	i->filename = StrDup(FileName);
-	if (!i->filename) {
-	    DBufFree(&buf);
-	    return E_NO_MEM;
-	}
+        i->filename = StrDup(FileName);
+        if (!i->filename) {
+            DBufFree(&buf);
+            return E_NO_MEM;
+        }
     } else {
-	i->filename = NULL;
+        i->filename = NULL;
     }
     i->ownedByMe = 1;
     i->LineNo = LineNo;
@@ -884,10 +905,12 @@ static int IncludeCmd(char const *cmd)
     i->offset = -1L;
     i->chain = NULL;
     if (fp) {
-	i->offset = ftell(fp);
-	FCLOSE(fp);
+        i->offset = ftell(fp);
+        FCLOSE(fp);
     }
     IStackPtr++;
+    NumIfs = 0;
+    IfFlags = 0;
 
     /* If the file is cached, use it */
     h = CachedFiles;
@@ -924,8 +947,9 @@ static int IncludeCmd(char const *cmd)
         fp2 = popen(cmd, "r");
     }
     if (!fp2) {
-	DBufFree(&buf);
-	return E_CANT_OPEN;
+        PopFile();
+        DBufFree(&buf);
+        return E_CANT_OPEN;
     }
     fp = fp2;
     LineNo = 0;
@@ -941,12 +965,12 @@ static int IncludeCmd(char const *cmd)
 
     DebugFlag = old_flag;
     if (r == OK) {
-	fp = NULL;
-	CLine = CachedFiles->cache;
-	LineNo = 0;
-	STRSET(FileName, fname);
-	DBufFree(&buf);
-	return OK;
+        fp = NULL;
+        CLine = CachedFiles->cache;
+        LineNo = 0;
+        STRSET(FileName, fname);
+        DBufFree(&buf);
+        return OK;
     }
     DBufFree(&buf);
     /* We failed */
@@ -968,16 +992,16 @@ int IncludeFile(char const *fname)
     int oldRunDisabled;
     struct stat statbuf;
 
-    FreshLine = 1;
+    got_a_fresh_line();
     clear_callstack();
-    if (IStackPtr+1 >= INCLUDE_NEST) return E_NESTED_INCLUDE;
+    if (IStackPtr >= INCLUDE_NEST) return E_NESTED_INCLUDE;
     i = &IStack[IStackPtr];
 
     if (FileName) {
-	i->filename = StrDup(FileName);
-	if (!i->filename) return E_NO_MEM;
+        i->filename = StrDup(FileName);
+        if (!i->filename) return E_NO_MEM;
     } else {
-	i->filename = NULL;
+        i->filename = NULL;
     }
     i->LineNo = LineNo;
     i->NumIfs = NumIfs;
@@ -987,66 +1011,68 @@ int IncludeFile(char const *fname)
     i->offset = -1L;
     i->chain = NULL;
     if (RunDisabled & RUN_NOTOWNER) {
-	i->ownedByMe = 0;
+        i->ownedByMe = 0;
     } else {
-	i->ownedByMe = 1;
+        i->ownedByMe = 1;
     }
     if (fp) {
-	i->offset = ftell(fp);
-	FCLOSE(fp);
+        i->offset = ftell(fp);
+        FCLOSE(fp);
     }
 
     IStackPtr++;
+    NumIfs = 0;
+    IfFlags = 0;
 
 #ifdef HAVE_GLOB
     /* If it's a directory, set up the glob chain here. */
     if (stat(fname, &statbuf) == 0) {
-	FilenameChain *fc;
-	if (S_ISDIR(statbuf.st_mode)) {
+        FilenameChain *fc;
+        if (S_ISDIR(statbuf.st_mode)) {
             /* Check safety */
             if (!CheckSafetyAux(&statbuf)) {
                 PopFile();
                 return E_NO_MATCHING_REMS;
             }
-	    if (SetupGlobChain(fname, i) == OK) { /* Glob succeeded */
-		if (!i->chain) { /* Oops... no matching files */
-		    if (!Hush) {
-			Eprint("%s: %s", fname, ErrMsg[E_NO_MATCHING_REMS]);
-		    }
-		    PopFile();
-		    return E_NO_MATCHING_REMS;
-		}
-		while(i->chain) {
-		    fc = i->chain;
-		    i->chain = i->chain->next;
+            if (SetupGlobChain(fname, i) == OK) { /* Glob succeeded */
+                if (!i->chain) { /* Oops... no matching files */
+                    if (!Hush) {
+                        Eprint("%s: %s", fname, GetErr(E_NO_MATCHING_REMS));
+                    }
+                    PopFile();
+                    return E_NO_MATCHING_REMS;
+                }
+                while(i->chain) {
+                    fc = i->chain;
+                    i->chain = i->chain->next;
 
-		    /* Munch first file */
-		    oldRunDisabled = RunDisabled;
-		    if (!OpenFile(fc->filename)) {
-			return OK;
-		    }
-		    Eprint("%s: %s", ErrMsg[E_CANT_OPEN], fc->filename);
-		    RunDisabled = oldRunDisabled;
-		}
-		/* Couldn't open anything... bail */
-		return PopFile();
-	    } else {
-		if (!Hush) {
-		    Eprint("%s: %s", fname, ErrMsg[E_NO_MATCHING_REMS]);
-		}
-	    }
-	    return E_NO_MATCHING_REMS;
-	}
+                    /* Munch first file */
+                    oldRunDisabled = RunDisabled;
+                    if (!OpenFile(fc->filename)) {
+                        return OK;
+                    }
+                    Eprint("%s: %s", GetErr(E_CANT_OPEN), fc->filename);
+                    RunDisabled = oldRunDisabled;
+                }
+                /* Couldn't open anything... bail */
+                return PopFile();
+            } else {
+                if (!Hush) {
+                    Eprint("%s: %s", fname, GetErr(E_NO_MATCHING_REMS));
+                }
+            }
+            return E_NO_MATCHING_REMS;
+        }
     }
 #endif
 
     oldRunDisabled = RunDisabled;
     /* Try to open the new file */
     if (!OpenFile(fname)) {
-	return OK;
+        return OK;
     }
     RunDisabled = oldRunDisabled;
-    Eprint("%s: %s", ErrMsg[E_CANT_OPEN], fname);
+    Eprint("%s: %s", GetErr(E_CANT_OPEN), fname);
     /* Ugh!  We failed!  */
     PopFile();
     return E_CANT_OPEN;
@@ -1066,9 +1092,9 @@ int GetAccessDate(char const *file)
     t1 = localtime(&(statbuf.st_atime));
 
     if (t1->tm_year + 1900 < BASE)
-	return 0;
+        return 0;
     else
-	return DSE(t1->tm_year+1900, t1->tm_mon, t1->tm_mday);
+        return DSE(t1->tm_year+1900, t1->tm_mon, t1->tm_mday);
 }
 
 /***************************************************************/
@@ -1085,21 +1111,21 @@ static void DestroyCache(CachedFile *cf)
     if (cf->filename) free((char *) cf->filename);
     cl = cf->cache;
     while (cl) {
-	if (cl->text) free ((char *) cl->text);
-	cnext = cl->next;
-	free(cl);
-	cl = cnext;
+        if (cl->text) free ((char *) cl->text);
+        cnext = cl->next;
+        free(cl);
+        cl = cnext;
     }
     if (CachedFiles == cf) CachedFiles = cf->next;
     else {
-	temp = CachedFiles;
-	while(temp) {
-	    if (temp->next == cf) {
-		temp->next = cf->next;
-		break;
-	    }
-	    temp = temp->next;
-	}
+        temp = CachedFiles;
+        while(temp) {
+            if (temp->next == cf) {
+                temp->next = cf->next;
+                break;
+            }
+            temp = temp->next;
+        }
     }
     free(cf);
 }
@@ -1133,13 +1159,19 @@ static int CheckSafety(void)
     struct stat statbuf;
 
     if (fp == stdin) {
-	return 1;
+        return 1;
     }
 
     if (fstat(fileno(fp), &statbuf)) {
-	fclose(fp);
-	fp = NULL;
-	return 0;
+        fclose(fp);
+        fp = NULL;
+        return 0;
+    }
+
+    if (S_ISDIR(statbuf.st_mode)) {
+        fclose(fp);
+        fp = NULL;
+        return 0;
     }
 
     if (!CheckSafetyAux(&statbuf)) {
@@ -1167,20 +1199,20 @@ static int CheckSafetyAux(struct stat *statbuf)
 {
     /* Under UNIX, take extra precautions if running as root */
     if (!geteuid()) {
-	/* Reject files not owned by root or group/world writable */
-	if (statbuf->st_uid != 0) {
-	    fprintf(ErrFp, "SECURITY: Won't read non-root-owned file or directory when running as root!\n");
-	    return 0;
-	}
+        /* Reject files not owned by root or group/world writable */
+        if (statbuf->st_uid != 0) {
+            fprintf(ErrFp, "SECURITY: Won't read non-root-owned file or directory when running as root!\n");
+            return 0;
+        }
     }
     /* Sigh... /dev/null is usually world-writable, so ignore devices,
        FIFOs, sockets, etc. */
     if (!S_ISREG(statbuf->st_mode) && !S_ISDIR(statbuf->st_mode)) {
-	return 1;
+        return 1;
     }
     if ((statbuf->st_mode & S_IWOTH)) {
-	fprintf(ErrFp, "SECURITY: Won't read world-writable file or directory!\n");
-	return 0;
+        fprintf(ErrFp, "SECURITY: Won't read world-writable file or directory!\n");
+        return 0;
     }
 
     /* If file is not owned by me or a trusted user, disable RUN command */
@@ -1189,7 +1221,7 @@ static int CheckSafetyAux(struct stat *statbuf)
     RunDisabled |= RUN_NOTOWNER;
     if (statbuf->st_uid == geteuid()) {
         /* Owned by me... safe */
-	RunDisabled &= ~RUN_NOTOWNER;
+        RunDisabled &= ~RUN_NOTOWNER;
     } else {
         int i;
         for (i=0; i<NumTrustedUsers; i++) {
