@@ -5,7 +5,7 @@
 /*  Function Prototypes.                                       */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2024 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2025 by Dianne Skoll                    */
 /*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
@@ -43,6 +43,7 @@
 int CallUserFunc (char const *name, int nargs, ParsePtr p);
 int DoFset (ParsePtr p);
 int DoFunset (ParsePtr p);
+int DoFrename (ParsePtr p);
 void UnsetAllUserFuncs(void);
 void ProduceCalendar (void);
 char const *SimpleTime (int tim);
@@ -55,12 +56,14 @@ int TriggerReminder (ParsePtr p, Trigger *t, TimeTrig *tim, int dse, int is_queu
 int ShouldTriggerReminder (Trigger *t, TimeTrig *tim, int dse, int *err);
 int DoSubst (ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int dse, int mode);
 int DoSubstFromString (char const *source, DynamicBuffer *dbuf, int dse, int tim);
-int ParseLiteralDate (char const **s, int *dse, int *tim);
+int ParseLiteralDateOrTime (char const **s, int *dse, int *tim);
 int ParseLiteralTime (char const **s, int *tim);
 expr_node *parse_expression(char const **e, int *r, Var *locals);
 
 int evaluate_expression(expr_node *node, Value *locals, Value *ans, int *nonconst);
 int evaluate_expr_node(expr_node *node, Value *locals, Value *ans, int *nonconst);
+int truthy(Value const *v);
+
 void print_expr_tree(expr_node *node, FILE *fp);
 void unlimit_execution_time(void);
 expr_node *free_expr_tree(expr_node *node);
@@ -84,6 +87,8 @@ void FromDSE (int dse, int *y, int *m, int *d);
 int JulianToGregorianOffset(int y, int m);
 int ParseChar (ParsePtr p, int *err, int peek);
 int ParseToken (ParsePtr p, DynamicBuffer *dbuf);
+int ParseQuotedString (ParsePtr p, DynamicBuffer *dbuf);
+int ParseTokenOrQuotedString (ParsePtr p, DynamicBuffer *dbuf);
 int ParseIdentifier (ParsePtr p, DynamicBuffer *dbuf);
 expr_node * ParseExpr(ParsePtr p, int *r);
 void print_expr_nodes_stats(void);
@@ -108,6 +113,9 @@ int DoDebug (ParsePtr p);
 int DoBanner (ParsePtr p);
 int DoRun (ParsePtr p);
 int DoExpr (ParsePtr p);
+int DoTranslate (ParsePtr p);
+int InsertTranslation(char const *orig, char const *translated);
+void DumpTranslationTable(FILE *fp, int json);
 int DoErrMsg (ParsePtr p);
 int ClearGlobalOmits (void);
 int DoClear (ParsePtr p);
@@ -155,7 +163,8 @@ int DoPreserve  (Parser *p);
 int DoSatRemind (Trigger *trig, TimeTrig *tt, ParsePtr p);
 int DoMsgCommand (char const *cmd, char const *msg, int is_queued);
 int ParseNonSpaceChar (ParsePtr p, int *err, int peek);
-unsigned int HashVal (char const *str);
+unsigned int HashVal_ignorecase(char const *str);
+unsigned int HashVal_preservecase(char const *str);
 int DateOK (int y, int m, int d);
 BuiltinFunc *FindBuiltinFunc (char const *name);
 int InsertIntoSortBuffer (int dse, int tim, char const *body, int typ, int prio);
@@ -220,7 +229,6 @@ void set_cloexec(FILE *fp);
 int push_call(char const *filename, char const *func, int lineno);
 void clear_callstack(void);
 int print_callstack(FILE *fp);
-int have_callstack(void);
 void pop_call(void);
 void FixSpecialType(Trigger *trig);
 void WriteJSONTrigger(Trigger const *t, int include_tags, int today);
@@ -240,3 +248,28 @@ void PutWideChar(wchar_t const wc, DynamicBuffer *output);
 extern int _private_mul_overflow(int a, int b);
 extern int _private_add_overflow(int a, int b);
 extern int _private_sub_overflow(int a, int b);
+
+/* Utility functions for dumping tokens */
+void print_sysvar_tokens(void);
+void print_builtinfunc_tokens(void);
+void print_remind_tokens(void);
+
+/* Stats for -ds output */
+void dump_var_hash_stats(void);
+void dump_userfunc_hash_stats(void);
+void dump_dedupe_hash_stats(void);
+void dump_translation_hash_stats(void);
+
+/* Dedupe code */
+int ShouldDedupe(int trigger_date, int trigger_time, char const *body);
+void ClearDedupeTable(void);
+void InitDedupeTable(void);
+
+void InitVars(void);
+void InitUserFunctions(void);
+void InitTranslationTable(void);
+char const *GetTranslatedString(char const *orig);
+int GetTranslatedStringTryingVariants(char const *orig, DynamicBuffer *out);
+char const *GetErr(int r);
+char const *tr(char const *s);
+void print_escaped_string(FILE *fp, char const *s);
