@@ -5,25 +5,30 @@
 /*  Type definitions all dumped here.                          */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2024 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2025 by Dianne Skoll                    */
 /*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
 
 #include <limits.h>
 #include "dynbuf.h"
+#include "hashtab.h"
 
 typedef struct udf_struct UserFunc;
 
-/* Define the types of values */
-#define ERR_TYPE       0
-#define INT_TYPE       1
-#define TIME_TYPE      2
-#define DATE_TYPE      3
-#define STR_TYPE       4
-#define DATETIME_TYPE  5
-#define SPECIAL_TYPE   6 /* Only for system variables */
-#define CONST_INT_TYPE 7 /* Only for system variables */
+/* Define the types of values.  We use bitmasks so we can define
+   DATETIME_TYPE as a combo of DATE_TYPE and TIME_TYPE */
+
+#define ERR_TYPE       0x0
+#define INT_TYPE       0x1
+#define TIME_TYPE      0x2
+#define DATE_TYPE      0x4
+/* DATETIME_TYPE has both DATE and TIME bits turned on */
+#define DATETIME_TYPE  (TIME_TYPE | DATE_TYPE)
+#define STR_TYPE       0x8
+#define SPECIAL_TYPE   0x10 /* Only for system variables */
+#define CONST_INT_TYPE 0x20 /* Only for system variables */
+#define TRANS_TYPE     0x40 /* Only for system variables */
 
 #define BEG_OF_EXPR '['
 #define END_OF_EXPR ']'
@@ -33,16 +38,15 @@ typedef struct udf_struct UserFunc;
 typedef struct {
     char type;
     union {
-	char *str;
-	int val;
+        char *str;
+        int val;
     } v;
 } Value;
 
 /* New-style expr_node structure and constants */
 enum expr_node_type
 {
-    N_FREE,
-    N_ERROR,
+    N_FREE = 0,
     N_CONSTANT,
     N_SHORT_STR,
     N_LOCAL_VAR,
@@ -54,6 +58,7 @@ enum expr_node_type
     N_SHORT_USER_FUNC,
     N_USER_FUNC,
     N_OPERATOR,
+    N_ERROR = 0x7FFF,
 };
 
 /* Structure for passing in Nargs and out RetVal from functions */
@@ -96,7 +101,7 @@ typedef struct expr_node_struct {
 
 /* Define the structure of a variable */
 typedef struct var {
-    struct var *next;
+    struct hash_link link;
     char name[VAR_NAME_LEN+1];
     char preserve;
     Value v;
@@ -146,12 +151,12 @@ typedef struct {
 
 /* The parse pointer */
 typedef struct {
-    DynamicBuffer pushedToken;	/* Pushed-back token */
+    DynamicBuffer pushedToken;  /* Pushed-back token */
     char const *text;           /* Start of text */
     char const *pos;            /* Current position */
     char const *etext;          /* Substituted text */
     char const *epos;           /* Position in substituted text */
-    char const *tokenPushed;	/* NULL if no pushed-back token */
+    char const *tokenPushed;    /* NULL if no pushed-back token */
     unsigned char isnested;      /* Is it a nested expression? */
     unsigned char allownested;
     unsigned char expr_happened; /* Did we encounter an [expression] ? */
@@ -195,66 +200,45 @@ typedef Parser *ParsePtr;  /* Pointer to parser structure */
 #define NO_MAX 127
 
 /* DEFINES for debugging flags */
-#define DB_PRTLINE      1
-#define DB_PRTEXPR      2
-#define DB_PRTTRIG      4
-#define DB_DUMP_VARS    8
-#define DB_ECHO_LINE   16
-#define DB_TRACE_FILES 32
-#define DB_PARSE_EXPR  64
+#define DB_PRTLINE      0x01
+#define DB_PRTEXPR      0x02
+#define DB_PRTTRIG      0x04
+#define DB_DUMP_VARS    0x08
+#define DB_ECHO_LINE    0x10
+#define DB_TRACE_FILES  0x20
+#define DB_PARSE_EXPR   0x40
+#define DB_HASHSTATS    0x80
 
 /* Enumeration of the tokens */
 enum TokTypes
 { T_Illegal,
-  /* Commands first */
-  T_Rem, T_Push, T_Pop, T_Preserve, T_Include, T_IncludeR, T_IncludeCmd, T_If, T_Else, T_EndIf,
-  T_IfTrig, T_ErrMsg,
-  T_Set, T_UnSet, T_Fset, T_Funset, T_Omit, T_Banner, T_Exit,
-  T_AddOmit, T_NoQueue,
-  T_WkDay,
-  T_Month, T_Time, T_Date, T_DateTime,
-  T_Skip, T_At, T_RemType, T_Until, T_Year, T_Day, T_Rep, T_Delta,
-  T_Back, T_BackAdj,
-  T_Once,
-  T_Empty,
-  T_Comment,
-  T_Number,
-  T_Clr,
-  T_Debug,
-  T_Dumpvars,
-  T_Scanfrom,
-  T_Flush,
-  T_Priority,
-  T_Sched,
-  T_Warn,
-  T_Tag,
-  T_Duration,
-  T_LongTime,
-  T_OmitFunc,
-  T_Through,
-  T_MaybeUncomputable,
-  T_Ordinal,
-  T_In,
-  T_LastBack,
-  T_Expr
+  T_AddOmit, T_At, T_Back, T_BackAdj, T_Banner, T_Clr, T_Comment,
+  T_Date, T_DateTime, T_Day, T_Debug, T_Delta, T_Dumpvars, T_Duration,
+  T_Else, T_Empty, T_EndIf, T_ErrMsg, T_Exit, T_Expr,
+  T_Flush, T_Frename, T_Fset, T_Funset, T_If, T_IfTrig, T_In,
+  T_Include, T_IncludeCmd, T_IncludeR, T_IncludeSys, T_LastBack, T_LongTime,
+  T_MaybeUncomputable, T_Month, T_NoQueue, T_Number, T_Omit, T_OmitFunc,
+  T_Once, T_Ordinal, T_Pop, T_Preserve, T_Priority, T_Push,T_Rem,
+  T_RemType, T_Rep, T_Scanfrom, T_Sched, T_Set, T_Skip, T_Tag, T_Through,
+  T_Time, T_Translate, T_UnSet, T_Until, T_Warn, T_WkDay, T_Year
 };
 
 /* The structure of a token */
 typedef struct {
-    char *name;
+    char const *name;
     char MinLen;
     enum TokTypes type;
     int val;
 } Token;
 
 /* Flags for the state of the "if" stack */
-#define IF_TRUE      0
-#define IF_FALSE     1
-#define BEFORE_ELSE  0
-#define AFTER_ELSE   2
-#define IF_MASK      3
-#define IF_TRUE_MASK 1
-#define IF_ELSE_MASK 2
+#define IF_TRUE      0x00
+#define IF_FALSE     0x01
+#define BEFORE_ELSE  0x00
+#define AFTER_ELSE   0x02
+#define IF_MASK      0x03
+#define IF_TRUE_MASK 0x01
+#define IF_ELSE_MASK 0x02
 
 /* Flags for the DoSubst function */
 #define NORMAL_MODE  0
@@ -264,9 +248,9 @@ typedef struct {
 #define QUOTE_MARKER 1 /* Unlikely character to appear in reminder */
 
 /* Flags for disabling run */
-#define RUN_CMDLINE  1
-#define RUN_SCRIPT   2
-#define RUN_NOTOWNER 4
+#define RUN_CMDLINE  0x01
+#define RUN_SCRIPT   0x02
+#define RUN_NOTOWNER 0x04
 
 /* Flags for the SimpleCalendar format */
 #define SC_AMPM   0   /* Time shown as 3:00am, etc. */
@@ -310,12 +294,12 @@ typedef struct {
 
 /* Define the data structure used to hold a user-defined function */
 typedef struct udf_struct {
-    struct udf_struct *next;
+    struct hash_link link;
     char name[VAR_NAME_LEN+1];
     expr_node *node;
     char **args;
     int nargs;
     char const *filename;
     int lineno;
+    int recurse_flag;
 } UserFunc;
-
