@@ -6,7 +6,7 @@
 /*  functions.                                                 */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2024 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2025 by Dianne Skoll                    */
 /*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
@@ -15,6 +15,7 @@
 
 #include <stdio.h>
 #include <ctype.h>
+#include <stddef.h>
 
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
@@ -27,36 +28,113 @@
 #include "protos.h"
 #include "err.h"
 
-#define FUNC_HASH_SIZE 32   /* Size of User-defined function hash table */
-
 /* The hash table */
-static UserFunc *FuncHash[FUNC_HASH_SIZE];
+hash_table FuncHash;
 
 static void DestroyUserFunc (UserFunc *f);
 static void FUnset (char const *name);
 static void FSet (UserFunc *f);
+static void RenameUserFunc(char const *oldname, char const *newname);
+
+static unsigned int HashUserFunc(void *x)
+{
+    UserFunc *f = (UserFunc *) x;
+    return HashVal_preservecase(f->name);
+}
+
+static int CompareUserFuncs(void *a, void *b)
+{
+    UserFunc *f = (UserFunc *) a;
+    UserFunc *g = (UserFunc *) b;
+    return strcmp(f->name, g->name);
+}
+
+void
+InitUserFunctions(void)
+{
+    if (hash_table_init(&FuncHash,
+                        offsetof(UserFunc, link),
+                        HashUserFunc,
+                        CompareUserFuncs) < 0) {
+        fprintf(ErrFp, "Unable to initialize function hash table: Out of memory.  Exiting.\n");
+        exit(1);
+    }
+}
 
 /***************************************************************/
 /*                                                             */
-/*  HashVal                                                    */
+/*  HashVal_preservecase                                       */
 /*  Given a string, compute the hash value.                    */
 /*                                                             */
 /***************************************************************/
-unsigned int HashVal_nocase(char const *str)
+unsigned int HashVal_preservecase(char const *str)
 {
-    register unsigned int i=0;
-    register unsigned int j=1;
-    register unsigned int len=0;
-
-    while(*str && len < VAR_NAME_LEN) {
-	i += j * (*str);
-	str++;
-	len++;
-	j = 3-j;
+    unsigned int h = 0, high;
+    while(*str) {
+        h = (h << 4) + (unsigned int) *str;
+        str++;
+        high = h & 0xF0000000;
+        if (high) {
+            h ^= (high >> 24);
+        }
+        h &= ~high;
     }
-    return i;
+    return h;
 }
 
+/***************************************************************/
+/*                                                             */
+/*  DoFrename                                                  */
+/*                                                             */
+/*  Rename a user-defined function - the FRENAME command.      */
+/*                                                             */
+/***************************************************************/
+int DoFrename(ParsePtr p)
+{
+    int r;
+    DynamicBuffer oldbuf;
+    DynamicBuffer newbuf;
+    DBufInit(&oldbuf);
+    DBufInit(&newbuf);
+
+    r = ParseIdentifier(p, &oldbuf);
+    if (r != OK) {
+        DBufFree(&oldbuf);
+        return r;
+    }
+    strtolower(DBufValue(&oldbuf));
+    r = ParseIdentifier(p, &newbuf);
+    if (r != OK) {
+        DBufFree(&oldbuf);
+        DBufFree(&newbuf);
+        return r;
+    }
+    strtolower(DBufValue(&newbuf));
+    r = VerifyEoln(p);
+    if (r != OK) {
+        DBufFree(&oldbuf);
+        DBufFree(&newbuf);
+        return r;
+    }
+    if (FindBuiltinFunc(DBufValue(&newbuf))) {
+        Eprint("%s: `%s'", GetErr(E_REDEF_FUNC), DBufValue(&newbuf));
+        DBufFree(&oldbuf);
+        DBufFree(&newbuf);
+        return E_REDEF_FUNC;
+    }
+    if (FindBuiltinFunc(DBufValue(&oldbuf))) {
+        Eprint("%s: `%s'", GetErr(E_REDEF_FUNC), DBufValue(&oldbuf));
+        DBufFree(&oldbuf);
+        DBufFree(&newbuf);
+        return E_REDEF_FUNC;
+    }
+    RenameUserFunc(DBufValue(&oldbuf), DBufValue(&newbuf));
+    DBufFree(&oldbuf);
+    DBufFree(&newbuf);
+    return OK;
+}
+
+
 /***************************************************************/
 /*                                                             */
 /*  DoFunset                                                   */
@@ -110,8 +188,8 @@ int DoFset(ParsePtr p)
     /* Get the function name */
     if ( (r=ParseIdentifier(p, &buf)) ) return r;
     if (*DBufValue(&buf) == '$') {
-	DBufFree(&buf);
-	return E_BAD_ID;
+        DBufFree(&buf);
+        return E_BAD_ID;
     }
     orig_namelen = buf.len;
 
@@ -129,23 +207,26 @@ int DoFset(ParsePtr p)
             /* We already have it!  Our work here is done. */
             return OK;
         }
+        /* Warn about redefinition */
+        Wprint("Function %s redefined (previously defined at %s:%d)",
+               existing->name, existing->filename, existing->lineno);
     }
 
     /* Should be followed by '(' */
     c = ParseNonSpaceChar(p, &r, 0);
     if (r) {
-	DBufFree(&buf);
-	return r;
+        DBufFree(&buf);
+        return r;
     }
     if (c != '(') {
-	DBufFree(&buf);
-	return E_PARSE_ERR;
+        DBufFree(&buf);
+        return E_PARSE_ERR;
     }
 
     func = NEW(UserFunc);
     if (!func) {
-	DBufFree(&buf);
-	return E_NO_MEM;
+        DBufFree(&buf);
+        return E_NO_MEM;
     }
     if (FileName) {
         func->filename = StrDup(FileName);
@@ -157,12 +238,13 @@ int DoFset(ParsePtr p)
         return E_NO_MEM;
     }
     func->lineno = LineNo;
+    func->recurse_flag = 0;
     StrnCpy(func->name, DBufValue(&buf), VAR_NAME_LEN);
     DBufFree(&buf);
     if (!Hush) {
-	if (FindBuiltinFunc(func->name)) {
-	    Eprint("%s: `%s'", ErrMsg[E_REDEF_FUNC], func->name);
-	}
+        if (FindBuiltinFunc(func->name)) {
+            Eprint("%s: `%s'", GetErr(E_REDEF_FUNC), func->name);
+        }
     }
     func->node = NULL;
     func->nargs = 0;
@@ -173,16 +255,16 @@ int DoFset(ParsePtr p)
     c=ParseNonSpaceChar(p, &r, 1);
     if (r) return r;
     if (c == ')') {
-	(void) ParseNonSpaceChar(p, &r, 0);
+        (void) ParseNonSpaceChar(p, &r, 0);
     } else {
         locals = local_array;
-	while(1) {
-	    if ( (r=ParseIdentifier(p, &buf)) ) return r;
-	    if (*DBufValue(&buf) == '$') {
-		DBufFree(&buf);
-		DestroyUserFunc(func);
-		return E_BAD_ID;
-	    }
+        while(1) {
+            if ( (r=ParseIdentifier(p, &buf)) ) return r;
+            if (*DBufValue(&buf) == '$') {
+                DBufFree(&buf);
+                DestroyUserFunc(func);
+                return E_BAD_ID;
+            }
             /* If we've already seen this local variable, error */
             for (i=0; i<func->nargs; i++) {
                 if (!StrinCmp(DBufValue(&buf), local_array[i].name, VAR_NAME_LEN)) {
@@ -199,27 +281,36 @@ int DoFset(ParsePtr p)
             }
             local_array[i].v.type = ERR_TYPE;
             StrnCpy(local_array[i].name, DBufValue(&buf), VAR_NAME_LEN);
-            local_array[i].next = &(local_array[i+1]);
-            local_array[i+1].next = NULL;
-	    func->nargs++;
-	    c = ParseNonSpaceChar(p, &r, 0);
-	    if (c == ')') break;
-	    else if (c != ',') {
-		DestroyUserFunc(func);
-		return E_PARSE_ERR;
-	    }
-	}
+            local_array[i].link.next = &(local_array[i+1]);
+            local_array[i+1].link.next = NULL;
+            func->nargs++;
+            c = ParseNonSpaceChar(p, &r, 0);
+            if (r) {
+                DBufFree(&buf);
+                DestroyUserFunc(func);
+                return r;
+            }
+            if (c == ')') break;
+            else if (c != ',') {
+                DestroyUserFunc(func);
+                return E_PARSE_ERR;
+            }
+        }
     }
 
     /* Allow an optional = sign: FSET f(x) = x*x */
     c = ParseNonSpaceChar(p, &r, 1);
+    if (r) {
+        DestroyUserFunc(func);
+        return r;
+    }
     if (c == '=') {
-	(void) ParseNonSpaceChar(p, &r, 0);
+        (void) ParseNonSpaceChar(p, &r, 0);
     }
     if (p->isnested) {
-	Eprint("%s", ErrMsg[E_CANTNEST_FDEF]);
-	DestroyUserFunc(func);
-	return E_PARSE_ERR;
+        Eprint("%s", GetErr(E_CANTNEST_FDEF));
+        DestroyUserFunc(func);
+        return E_PARSE_ERR;
     }
 
     while(*(p->pos) && isspace(*(p->pos))) {
@@ -237,14 +328,15 @@ int DoFset(ParsePtr p)
     }
 
     c = ParseNonSpaceChar(p, &r, 1);
-    if (c != 0) {
+    if (c != 0 || r != 0) {
         DestroyUserFunc(func);
+        if (r != 0) return r;
         return E_EXPECTING_EOL;
     }
 
     /* Save the argument names */
     if (func->nargs) {
-        func->args = calloc(sizeof(char *), func->nargs);
+        func->args = calloc(func->nargs, sizeof(char *));
         for (i=0; i<func->nargs; i++) {
             func->args[i] = StrDup(local_array[i].name);
             if (!func->args[i]) {
@@ -260,8 +352,8 @@ int DoFset(ParsePtr p)
     /* Add the function definition */
     FSet(func);
     if (orig_namelen > VAR_NAME_LEN) {
-	Wprint("Warning: Function name `%s...' truncated to `%s'",
-	       func->name, func->name);
+        Wprint("Warning: Function name `%s...' truncated to `%s'",
+               func->name, func->name);
     }
     return OK;
 }
@@ -305,21 +397,11 @@ static void DestroyUserFunc(UserFunc *f)
 /***************************************************************/
 static void FUnset(char const *name)
 {
-    UserFunc *cur, *prev;
-    int h;
-
-    h = HashVal_nocase(name) % FUNC_HASH_SIZE;
-
-    cur = FuncHash[h];
-    prev = NULL;
-    while(cur) {
-	if (! strncmp(name, cur->name, VAR_NAME_LEN)) break;
-	prev = cur;
-	cur = cur->next;
+    UserFunc *f = FindUserFunc(name);
+    if (f) {
+        hash_table_delete(&FuncHash, f);
+        DestroyUserFunc(f);
     }
-    if (!cur) return;
-    if (prev) prev->next = cur->next; else FuncHash[h] = cur->next;
-    DestroyUserFunc(cur);
 }
 
 /***************************************************************/
@@ -331,19 +413,17 @@ static void FUnset(char const *name)
 /***************************************************************/
 static void FSet(UserFunc *f)
 {
-    int h = HashVal_nocase(f->name) % FUNC_HASH_SIZE;
-    f->next = FuncHash[h];
-    FuncHash[h] = f;
+    hash_table_insert(&FuncHash, f);
 }
 
 UserFunc *FindUserFunc(char const *name)
 {
    UserFunc *f;
-   int h = HashVal_nocase(name) % FUNC_HASH_SIZE;
+   UserFunc candidate;
 
-   /* Search for the function */
-   f = FuncHash[h];
-   while (f && strncmp(name, f->name, VAR_NAME_LEN)) f = f->next;
+   StrnCpy(candidate.name, name, VAR_NAME_LEN);
+
+   f = hash_table_find(&FuncHash, &candidate);
    return f;
 }
 
@@ -376,14 +456,58 @@ UnsetAllUserFuncs(void)
 {
     UserFunc *f;
     UserFunc *next;
-    int i;
-    for (i=0; i<FUNC_HASH_SIZE; i++) {
-        f = FuncHash[i];
-        while(f) {
-            next = f->next;
-            DestroyUserFunc(f);
-            f = next;
-        }
-        FuncHash[i] = NULL;
+
+    f = hash_table_next(&FuncHash, NULL);
+    while(f) {
+        next = hash_table_next(&FuncHash, f);
+        hash_table_delete_no_resize(&FuncHash, f);
+        DestroyUserFunc(f);
+        f = next;
     }
+    hash_table_free(&FuncHash);
+    InitUserFunctions();
 }
+
+/***************************************************************/
+/*                                                             */
+/*  RenameUserFunc                                             */
+/*                                                             */
+/*  Rename a user-defined function.                            */
+/*  If newname exists, it is deleted.                          */
+/*  If oldname exists, it is renamed to newname.               */
+/*                                                             */
+/***************************************************************/
+static void
+RenameUserFunc(char const *oldname, char const *newname)
+{
+    UserFunc *f = FindUserFunc(oldname);
+
+    if (!strcmp(oldname, newname)) {
+        /* Same name; do nothing */
+        return;
+    }
+
+    /* Unset newname */
+    FUnset(newname);
+
+    /* If oldname does not exist, we're done. */
+    if (!f) {
+        return;
+    }
+
+    /* Remove from hash table */
+    hash_table_delete(&FuncHash, f);
+
+    /* Rename */
+    StrnCpy(f->name, newname, VAR_NAME_LEN);
+
+    /* Insert into hash table */
+    hash_table_insert(&FuncHash, f);
+}
+
+void
+dump_userfunc_hash_stats(void)
+{
+    hash_table_dump_stats(&FuncHash, ErrFp);
+}
+
