@@ -6,7 +6,7 @@
 /*  user- and system-defined variables.                        */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2024 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2025 by Dianne Skoll                    */
 /*  SPDX-License-Identifier: GPL-2.0-only                      */
 /*                                                             */
 /***************************************************************/
@@ -17,6 +17,7 @@
 #include <string.h>
 #include <ctype.h>
 
+#include <stddef.h>
 #include <stdlib.h>
 #include <limits.h>
 #include <errno.h>
@@ -27,16 +28,38 @@
 #include "err.h"
 #define UPPER(c) (islower(c) ? toupper(c) : c)
 
-/* The variable hash table */
-#define VAR_HASH_SIZE 64
-#define VARIABLE ErrMsg[E_VAR]
-#define VALUE    ErrMsg[E_VAL]
-#define UNDEF	 ErrMsg[E_UNDEF]
+#define VARIABLE GetErr(E_VAR)
+#define VALUE    GetErr(E_VAL)
+#define UNDEF    GetErr(E_UNDEF)
 
 static int IntMin = INT_MIN;
 static int IntMax = INT_MAX;
 
-static Var *VHashTbl[VAR_HASH_SIZE];
+static hash_table VHashTbl;
+static int SetSysVarHelper(SysVar *v, Value *value);
+
+static unsigned int VarHashFunc(void *x)
+{
+    Var *v = (Var *) x;
+    return HashVal_ignorecase(v->name);
+}
+
+static int VarCompareFunc(void *a, void *b)
+{
+    Var *x = (Var *) a;
+    Var *y = (Var *) b;
+    return StrCmpi(x->name, y->name);
+}
+
+void
+InitVars(void)
+{
+    if (hash_table_init(&VHashTbl, offsetof(Var, link),
+                        VarHashFunc, VarCompareFunc) < 0) {
+        fprintf(ErrFp, "Unable to initialize variable hash table: Out of memory.  Exiting.\n");
+        exit(1);
+    }
+}
 
 static double
 strtod_in_c_locale(char const *str, char **endptr)
@@ -224,10 +247,10 @@ static int trig_date_func(int do_set, Value *val)
     UNUSED(do_set);
     if (!LastTrigValid) {
         val->type = INT_TYPE;
-	val->v.val = 0;
+        val->v.val = 0;
     } else {
         val->type = DATE_TYPE;
-	val->v.val = LastTriggerDate;
+        val->v.val = LastTriggerDate;
     }
     return OK;
 }
@@ -237,8 +260,8 @@ static int trig_day_func(int do_set, Value *val)
     UNUSED(do_set);
     val->type = INT_TYPE;
     if (!LastTrigValid) {
-	val->v.val = -1;
-	return OK;
+        val->v.val = -1;
+        return OK;
     }
 
     FromDSE(LastTriggerDate, &y, &m, &d);
@@ -252,8 +275,8 @@ static int trig_mon_func(int do_set, Value *val)
     UNUSED(do_set);
     val->type = INT_TYPE;
     if (!LastTrigValid) {
-	val->v.val = -1;
-	return OK;
+        val->v.val = -1;
+        return OK;
     }
 
     FromDSE(LastTriggerDate, &y, &m, &d);
@@ -267,8 +290,8 @@ static int trig_year_func(int do_set, Value *val)
     UNUSED(do_set);
     val->type = INT_TYPE;
     if (!LastTrigValid) {
-	val->v.val = -1;
-	return OK;
+        val->v.val = -1;
+        return OK;
     }
 
     FromDSE(LastTriggerDate, &y, &m, &d);
@@ -281,8 +304,8 @@ static int trig_wday_func(int do_set, Value *val)
     val->type = INT_TYPE;
     UNUSED(do_set);
     if (!LastTrigValid) {
-	val->v.val = -1;
-	return OK;
+        val->v.val = -1;
+        return OK;
     }
 
     val->v.val = (LastTriggerDate + 1) % 7;
@@ -337,17 +360,17 @@ static int today_wday_func(int do_set, Value *val)
 static int datetime_sep_func(int do_set, Value *val)
 {
     if (!do_set) {
-	val->v.str = malloc(2);
-	if (!val->v.str) return E_NO_MEM;
-	val->v.str[0] = DateTimeSep;
-	val->v.str[1] = 0;
-	val->type = STR_TYPE;
-	return OK;
+        val->v.str = malloc(2);
+        if (!val->v.str) return E_NO_MEM;
+        val->v.str[0] = DateTimeSep;
+        val->v.str[1] = 0;
+        val->type = STR_TYPE;
+        return OK;
     }
     if (val->type != STR_TYPE) return E_BAD_TYPE;
     if (strcmp(val->v.str, "T") &&
-	strcmp(val->v.str, "@")) {
-	return E_BAD_TYPE;
+        strcmp(val->v.str, "@")) {
+        return E_BAD_TYPE;
     }
     DateTimeSep = val->v.str[0];
     return OK;
@@ -397,10 +420,10 @@ static int default_color_func(int do_set, Value *val)
     }
     /* They either all have to be -1, or all between 0 and 255 */
     if (col_r == -1 && col_g == -1 && col_b == -1) {
-	DefaultColorR = -1;
-	DefaultColorG = -1;
-	DefaultColorB = -1;
-	return OK;
+        DefaultColorR = -1;
+        DefaultColorG = -1;
+        DefaultColorB = -1;
+        return OK;
     }
     if (col_r < 0) return E_2LOW;
     if (col_r > 255) return E_2HIGH;
@@ -418,17 +441,17 @@ static int default_color_func(int do_set, Value *val)
 static int date_sep_func(int do_set, Value *val)
 {
     if (!do_set) {
-	val->v.str = malloc(2);
-	if (!val->v.str) return E_NO_MEM;
-	val->v.str[0] = DateSep;
-	val->v.str[1] = 0;
-	val->type = STR_TYPE;
-	return OK;
+        val->v.str = malloc(2);
+        if (!val->v.str) return E_NO_MEM;
+        val->v.str[0] = DateSep;
+        val->v.str[1] = 0;
+        val->type = STR_TYPE;
+        return OK;
     }
     if (val->type != STR_TYPE) return E_BAD_TYPE;
     if (strcmp(val->v.str, "/") &&
-	strcmp(val->v.str, "-")) {
-	return E_BAD_TYPE;
+        strcmp(val->v.str, "-")) {
+        return E_BAD_TYPE;
     }
     DateSep = val->v.str[0];
     return OK;
@@ -437,17 +460,17 @@ static int date_sep_func(int do_set, Value *val)
 static int time_sep_func(int do_set, Value *val)
 {
     if (!do_set) {
-	val->v.str = malloc(2);
-	if (!val->v.str) return E_NO_MEM;
-	val->v.str[0] = TimeSep;
-	val->v.str[1] = 0;
-	val->type = STR_TYPE;
-	return OK;
+        val->v.str = malloc(2);
+        if (!val->v.str) return E_NO_MEM;
+        val->v.str[0] = TimeSep;
+        val->v.str[1] = 0;
+        val->type = STR_TYPE;
+        return OK;
     }
     if (val->type != STR_TYPE) return E_BAD_TYPE;
     if (strcmp(val->v.str, ":") &&
-	strcmp(val->v.str, ".")) {
-	return E_BAD_TYPE;
+        strcmp(val->v.str, ".")) {
+        return E_BAD_TYPE;
     }
     TimeSep = val->v.str[0];
     return OK;
@@ -455,23 +478,23 @@ static int time_sep_func(int do_set, Value *val)
 
 /***************************************************************/
 /*                                                             */
-/*  HashVal                                                    */
-/*  Given a string, compute the hash value.                    */
+/*  HashVal_ignorecase                                         */
+/*  Given a string, compute the hash value case-insensitively  */
 /*                                                             */
 /***************************************************************/
-unsigned int HashVal(char const *str)
+unsigned int HashVal_ignorecase(char const *str)
 {
-    register unsigned int i=0;
-    register unsigned int j=1;
-    register unsigned int len=0;
-
-    while(*str && len < VAR_NAME_LEN) {
-	i += j * (unsigned int) UPPER(*str);
-	str++;
-	len++;
-	j = 3-j;
+    unsigned int h = 0, high;
+    while(*str) {
+        h = (h << 4) + (unsigned int) UPPER(*str);
+        str++;
+        high = h & 0xF0000000;
+        if (high) {
+            h ^= (high >> 24);
+        }
+        h &= ~high;
     }
-    return i;
+    return h;
 }
 
 /***************************************************************/
@@ -483,31 +506,22 @@ unsigned int HashVal(char const *str)
 /***************************************************************/
 Var *FindVar(char const *str, int create)
 {
-    register int h;
-    register Var *v;
-    register Var *prev;
+    Var *v;
+    Var candidate;
+    StrnCpy(candidate.name, str, VAR_NAME_LEN);
 
-    h = HashVal(str) % VAR_HASH_SIZE;
-    v = VHashTbl[h];
-    prev = NULL;
+    v = (Var *) hash_table_find(&VHashTbl, &candidate);
+    if (v != NULL || !create) return v;
 
-    while(v) {
-	if (! StrinCmp(str, v->name, VAR_NAME_LEN)) return v;
-	prev = v;
-	v = v-> next;
-    }
-    if (!create) return v;
-
-/* Create the variable */
+    /* Create the variable */
     v = NEW(Var);
     if (!v) return v;
-    v->next = NULL;
     v->v.type = INT_TYPE;
     v->v.v.val = 0;
     v->preserve = 0;
     StrnCpy(v->name, str, VAR_NAME_LEN);
 
-    if (prev) prev->next = v; else VHashTbl[h] = v;
+    hash_table_insert(&VHashTbl, v);
     return v;
 }
 
@@ -520,23 +534,12 @@ Var *FindVar(char const *str, int create)
 /***************************************************************/
 int DeleteVar(char const *str)
 {
-    register int h;
-    register Var *v;
-    register Var *prev;
+    Var *v;
 
-    h = HashVal(str) % VAR_HASH_SIZE;
-    v = VHashTbl[h];
-    prev = NULL;
-
-    while(v) {
-	if (! StrinCmp(str, v->name, VAR_NAME_LEN)) break;
-	prev = v;
-	v = v-> next;
-    }
+    v = FindVar(str, 0);
     if (!v) return E_NOSUCH_VAR;
     DestroyValue(v->v);
-    if (prev) prev->next = v->next; else VHashTbl[h] = v->next;
-    free(v);
+    hash_table_delete(&VHashTbl, v);
     return OK;
 }
 
@@ -572,8 +575,8 @@ int GetVarValue(char const *str, Value *val)
     v=FindVar(str, 0);
 
     if (!v) {
-	Eprint("%s: %s", ErrMsg[E_NOSUCH_VAR], str);
-	return E_NOSUCH_VAR;
+        Eprint("%s: `%s'", GetErr(E_NOSUCH_VAR), str);
+        return E_NOSUCH_VAR;
     }
     return CopyValue(val, &v->v);
 }
@@ -587,7 +590,7 @@ int DoSet (Parser *p)
 {
     Value v;
     int r;
-
+    int ch;
     DynamicBuffer buf;
     DynamicBuffer buf2;
     DBufInit(&buf);
@@ -597,8 +600,11 @@ int DoSet (Parser *p)
     if (r) return r;
 
     /* Allow optional equals-sign:  SET var = value */
-    if (ParseNonSpaceChar(p, &r, 1) == '=') {
-	ParseNonSpaceChar(p, &r, 0);
+    ch = ParseNonSpaceChar(p, &r, 1);
+    if (r) return r;
+    if (ch == '=') {
+        ParseNonSpaceChar(p, &r, 0);
+        if (r) return r;
     }
 
     if (p->isnested) {
@@ -608,8 +614,8 @@ int DoSet (Parser *p)
 
     r = EvaluateExpr(p, &v);
     if (r) {
-	DBufFree(&buf);
-	return r;
+        DBufFree(&buf);
+        return r;
     }
 
     r = ParseToken(p, &buf2);
@@ -622,8 +628,8 @@ int DoSet (Parser *p)
     if (*DBufValue(&buf) == '$') r = SetSysVar(DBufValue(&buf)+1, &v);
     else r = SetVar(DBufValue(&buf), &v);
     if (buf.len > VAR_NAME_LEN) {
-	Wprint("Warning: Variable name `%.*s...' truncated to `%.*s'",
-	       VAR_NAME_LEN, DBufValue(&buf), VAR_NAME_LEN, DBufValue(&buf));
+        Wprint("Warning: Variable name `%.*s...' truncated to `%.*s'",
+               VAR_NAME_LEN, DBufValue(&buf), VAR_NAME_LEN, DBufValue(&buf));
     }
     DBufFree(&buf);
     return r;
@@ -644,21 +650,21 @@ int DoUnset (Parser *p)
     r = ParseToken(p, &buf);
     if (r) return r;
     if (!DBufLen(&buf)) {
-	DBufFree(&buf);
-	return E_EOLN;
+        DBufFree(&buf);
+        return E_EOLN;
     }
 
     (void) DeleteVar(DBufValue(&buf));  /* Ignore error - nosuchvar */
 
 /* Keep going... */
     while(1) {
-	r = ParseToken(p, &buf);
-	if (r) return r;
-	if (!DBufLen(&buf)) {
-	    DBufFree(&buf);
-	    return OK;
-	}
-	(void) DeleteVar(DBufValue(&buf));
+        r = ParseToken(p, &buf);
+        if (r) return r;
+        if (!DBufLen(&buf)) {
+            DBufFree(&buf);
+            return OK;
+        }
+        (void) DeleteVar(DBufValue(&buf));
     }
 }
 
@@ -681,35 +687,35 @@ int DoDump(ParsePtr p)
     r = ParseToken(p, &buf);
     if (r) return r;
     if (!*DBufValue(&buf) ||
-	*DBufValue(&buf) == '#' ||
-	*DBufValue(&buf) == ';') {
-	DBufFree(&buf);
-	DumpVarTable();
-	return OK;
+        *DBufValue(&buf) == '#' ||
+        *DBufValue(&buf) == ';') {
+        DBufFree(&buf);
+        DumpVarTable();
+        return OK;
     }
     fprintf(ErrFp, "%s  %s\n\n", VARIABLE, VALUE);
     while(1) {
-	if (*DBufValue(&buf) == '$') {
-	    DumpSysVarByName(DBufValue(&buf)+1);
-	} else {
-	    v = FindVar(DBufValue(&buf), 0);
-	    DBufValue(&buf)[VAR_NAME_LEN] = 0;
-	    if (!v) fprintf(ErrFp, "%s  %s\n",
-			    DBufValue(&buf), UNDEF);
-	    else {
-		fprintf(ErrFp, "%s  ", v->name);
-		PrintValue(&(v->v), ErrFp);
-		fprintf(ErrFp, "\n");
-	    }
-	}
-	r = ParseToken(p, &buf);
-	if (r) return r;
-	if (!*DBufValue(&buf) ||
-	    *DBufValue(&buf) == '#' ||
-	    *DBufValue(&buf) == ';') {
-	    DBufFree(&buf);
-	    return OK;
-	}
+        if (*DBufValue(&buf) == '$') {
+            DumpSysVarByName(DBufValue(&buf)+1);
+        } else {
+            v = FindVar(DBufValue(&buf), 0);
+            DBufValue(&buf)[VAR_NAME_LEN] = 0;
+            if (!v) fprintf(ErrFp, "%s  %s\n",
+                            DBufValue(&buf), UNDEF);
+            else {
+                fprintf(ErrFp, "%s  ", v->name);
+                PrintValue(&(v->v), ErrFp);
+                fprintf(ErrFp, "\n");
+            }
+        }
+        r = ParseToken(p, &buf);
+        if (r) return r;
+        if (!*DBufValue(&buf) ||
+            *DBufValue(&buf) == '#' ||
+            *DBufValue(&buf) == ';') {
+            DBufFree(&buf);
+            return OK;
+        }
     }
 }
 
@@ -717,24 +723,19 @@ int DoDump(ParsePtr p)
 /*                                                             */
 /*  DumpVarTable                                               */
 /*                                                             */
-/*  Dump the variable table to stderr.                         */
+/*  Dump the variable table to ErrFp.                         */
 /*                                                             */
 /***************************************************************/
 void DumpVarTable(void)
 {
-    register Var *v;
-    register int i;
+    Var *v;
 
     fprintf(ErrFp, "%s  %s\n\n", VARIABLE, VALUE);
 
-    for (i=0; i<VAR_HASH_SIZE; i++) {
-	v = VHashTbl[i];
-	while(v) {
-	    fprintf(ErrFp, "%s  ", v->name);
-	    PrintValue(&(v->v), ErrFp);
-	    fprintf(ErrFp, "\n");
-	    v = v->next;
-	}
+    hash_table_for_each(v, &VHashTbl) {
+        fprintf(ErrFp, "%s  ", v->name);
+        PrintValue(&(v->v), ErrFp);
+        fprintf(ErrFp, "\n");
     }
 }
 
@@ -748,27 +749,22 @@ void DumpVarTable(void)
 /***************************************************************/
 void DestroyVars(int all)
 {
-    int i;
-    Var *v, *next, *prev;
+    Var *v;
+    Var *next;
 
-    for (i=0; i<VAR_HASH_SIZE; i++) {
-	v = VHashTbl[i];
-	VHashTbl[i] = NULL;
-	prev = NULL;
-	while(v) {
-	    if (all || !v->preserve) {
-		DestroyValue(v->v);
-		next = v->next;
-		free(v);
-	    } else {
-		if (prev) prev->next = v;
-		else VHashTbl[i] = v;
-		prev = v;
-		next = v->next;
-		v->next = NULL;
-	    }
-	    v = next;
-	}
+    v = hash_table_next(&VHashTbl, NULL);
+    while(v) {
+        next = hash_table_next(&VHashTbl, v);
+        if (all || !v->preserve) {
+            DestroyValue(v->v);
+            hash_table_delete_no_resize(&VHashTbl, v);
+            free(v);
+        }
+        v = next;
+    }
+    if (all) {
+        hash_table_free(&VHashTbl);
+        InitVars();
     }
 }
 
@@ -804,8 +800,8 @@ int DoPreserve (Parser *p)
     r = ParseToken(p, &buf);
     if (r) return r;
     if (!DBufLen(&buf)) {
-	DBufFree(&buf);
-	return E_EOLN;
+        DBufFree(&buf);
+        return E_EOLN;
     }
 
     r = PreserveVar(DBufValue(&buf));
@@ -814,15 +810,15 @@ int DoPreserve (Parser *p)
 
 /* Keep going... */
     while(1) {
-	r = ParseToken(p, &buf);
-	if (r) return r;
-	if (!DBufLen(&buf)) {
-	    DBufFree(&buf);
-	    return OK;
-	}
-	r = PreserveVar(DBufValue(&buf));
-	DBufFree(&buf);
-	if (r) return r;
+        r = ParseToken(p, &buf);
+        if (r) return r;
+        if (!DBufLen(&buf)) {
+            DBufFree(&buf);
+            return OK;
+        }
+        r = PreserveVar(DBufValue(&buf));
+        DBufFree(&buf);
+        if (r) return r;
     }
 }
 
@@ -849,45 +845,48 @@ int DoPreserve (Parser *p)
 static SysVar SysVarArr[] = {
     /*  name          mod  type              value          min/mal   max */
     {"AddBlankLines",  1,  INT_TYPE,     &AddBlankLines,       0,      1 },
-    {"Ago",            1,  STR_TYPE,     &DynamicAgo,          0,      0 },
-    {"Am",             1,  STR_TYPE,     &DynamicAm,           0,      0 },
-    {"And",            1,  STR_TYPE,     &DynamicAnd,          0,      0 },
-    {"April",          1,  STR_TYPE,     &DynamicMonthName[3], 0,      0 },
-    {"At",             1,  STR_TYPE,     &DynamicAt,           0,      0 },
-    {"August",         1,  STR_TYPE,     &DynamicMonthName[7], 0,      0 },
+    {"Ago",            1,  TRANS_TYPE,   "ago",                0,      0 },
+    {"Am",             1,  TRANS_TYPE,   "am",                 0,      0 },
+    {"And",            1,  TRANS_TYPE,   "and",                0,      0 },
+    {"April",          1,  TRANS_TYPE,   "April",              0,      0 },
+    {"At",             1,  TRANS_TYPE,   "at",                 0,      0 },
+    {"August",         1,  TRANS_TYPE,   "August",             0,      0 },
     {"CalcUTC",        1,  INT_TYPE,     &CalculateUTC,        0,      1 },
     {"CalMode",        0,  INT_TYPE,     &DoCalendar,          0,      0 },
+    {"CalType",        0,  STR_TYPE,     &CalType,             0,      0 },
     {"Daemon",         0,  INT_TYPE,     &Daemon,              0,      0 },
     {"DateSep",        1,  SPECIAL_TYPE, date_sep_func,        0,      0 },
     {"DateTimeSep",    1,  SPECIAL_TYPE, datetime_sep_func,    0,      0 },
-    {"December",       1,  STR_TYPE,     &DynamicMonthName[11],0,      0 },
+    {"December",       1,  TRANS_TYPE,   "December",           0,      0 },
+    {"DedupeReminders",1,  INT_TYPE,     &DedupeReminders,     0,      1 },
     {"DefaultColor",   1,  SPECIAL_TYPE, default_color_func,   0,      0 },
+    {"DefaultDelta",   1,  INT_TYPE,     &DefaultDelta,        0,      10000 },
     {"DefaultPrio",    1,  INT_TYPE,     &DefaultPrio,         0,      9999 },
     {"DefaultTDelta",  1,  INT_TYPE,     &DefaultTDelta,       0,      1440 },
-    {"DeltaOverride",  0,  INT_TYPE,     &DeltaOverride,         0,      0 },
+    {"DeltaOverride",  0,  INT_TYPE,     &DeltaOverride,       0,      0 },
     {"DontFork",       0,  INT_TYPE,     &DontFork,            0,      0 },
     {"DontQueue",      0,  INT_TYPE,     &DontQueue,           0,      0 },
     {"DontTrigAts",    0,  INT_TYPE,     &DontIssueAts,        0,      0 },
     {"EndSent",        1,  STR_TYPE,     &EndSent,             0,      0 },
     {"EndSentIg",      1,  STR_TYPE,     &EndSentIg,           0,      0 },
     {"ExpressionTimeLimit", 1, SPECIAL_TYPE, expr_time_limit_func, 0,  0 },
-    {"February",       1,  STR_TYPE,     &DynamicMonthName[1], 0,      0 },
+    {"February",       1,  TRANS_TYPE,   "February",           0,      0 },
     {"FirstIndent",    1,  INT_TYPE,     &FirstIndent,         0,      132 },
     {"FoldYear",       1,  INT_TYPE,     &FoldYear,            0,      1 },
     {"FormWidth",      1,  INT_TYPE,     &FormWidth,           20,     500 },
-    {"Friday",         1,  STR_TYPE,     &DynamicDayName[4],   0,      0 },
-    {"Fromnow",        1,  STR_TYPE,     &DynamicFromnow,      0,      0 },
-    {"Hour",           1,  STR_TYPE,     &DynamicHour,         0,      0 },
+    {"Friday",         1,  TRANS_TYPE,   "Friday",             0,      0 },
+    {"Fromnow",        1,  TRANS_TYPE,   "from now",           0,      0 },
+    {"Hour",           1,  TRANS_TYPE,   "hour",               0,      0 },
     {"Hplu",           1,  STR_TYPE,     &DynamicHplu,         0,      0 },
     {"HushMode",       0,  INT_TYPE,     &Hush,                0,      0 },
     {"IgnoreOnce",     0,  INT_TYPE,     &IgnoreOnce,          0,      0 },
     {"InfDelta",       0,  INT_TYPE,     &InfiniteDelta,       0,      0 },
     {"IntMax",         0,  INT_TYPE,     &IntMax,              0,      0 },
     {"IntMin",         0,  INT_TYPE,     &IntMin,              0,      0 },
-    {"Is",             1,  STR_TYPE,     &DynamicIs,           0,      0 },
-    {"January",        1,  STR_TYPE,     &DynamicMonthName[0], 0,      0 },
-    {"July",           1,  STR_TYPE,     &DynamicMonthName[6], 0,      0 },
-    {"June",           1,  STR_TYPE,     &DynamicMonthName[5], 0,      0 },
+    {"Is",             1,  TRANS_TYPE,   "is",                 0,      0 },
+    {"January",        1,  TRANS_TYPE,   "January",            0,      0 },
+    {"July",           1,  TRANS_TYPE,   "July",               0,      0 },
+    {"June",           1,  TRANS_TYPE,   "June",               0,      0 },
     {"LatDeg",         1,  SPECIAL_TYPE, latdeg_func,          0,      0 },
     {"Latitude",       1,  SPECIAL_TYPE, latitude_func,        0,      0 },
     {"LatMin",         1,  SPECIAL_TYPE, latmin_func,          0,      0 },
@@ -897,52 +896,53 @@ static SysVar SysVarArr[] = {
     {"Longitude",      1,  SPECIAL_TYPE, longitude_func,       0,      0 },
     {"LongMin",        1,  SPECIAL_TYPE, longmin_func,         0,      0 },
     {"LongSec",        1,  SPECIAL_TYPE, longsec_func,         0,      0 },
-    {"March",          1,  STR_TYPE,     &DynamicMonthName[2], 0,      0 },
+    {"March",          1,  TRANS_TYPE,   "March",              0,      0 },
     {"MaxFullOmits",   0,  CONST_INT_TYPE, NULL,        MAX_FULL_OMITS, 0},
     {"MaxLateMinutes", 1,  INT_TYPE,     &MaxLateMinutes,      0,      1440 },
     {"MaxPartialOmits",0,  CONST_INT_TYPE, NULL,    MAX_PARTIAL_OMITS, 0},
     {"MaxSatIter",     1,  INT_TYPE,     &MaxSatIter,          10,     ANY },
     {"MaxStringLen",   1,  INT_TYPE,     &MaxStringLen,        -1,     ANY },
-    {"May",            1,  STR_TYPE,     &DynamicMonthName[4], 0,      0 },
+    {"May",            1,  TRANS_TYPE,   "May",                0,      0 },
     {"MinsFromUTC",    1,  INT_TYPE,     &MinsFromUTC,         -780,   780 },
-    {"Minute",         1,  STR_TYPE,     &DynamicMinute,       0,      0 },
-    {"Monday",         1,  STR_TYPE,     &DynamicDayName[0],   0,      0 },
+    {"Minute",         1,  TRANS_TYPE,   "minute",             0,      0 },
+    {"Monday",         1,  TRANS_TYPE,   "Monday",             0,      0 },
     {"Mplu",           1,  STR_TYPE,     &DynamicMplu,         0,      0 },
     {"NextMode",       0,  INT_TYPE,     &NextMode,            0,      0 },
-    {"November",       1,  STR_TYPE,     &DynamicMonthName[10],0,      0 },
-    {"Now",            1,  STR_TYPE,     &DynamicNow,          0,      0 },
+    {"November",       1,  TRANS_TYPE,   "November",           0,      0 },
+    {"Now",            1,  TRANS_TYPE,   "now",                0,      0 },
     {"NumFullOmits",   0,  INT_TYPE,     &NumFullOmits,        0,      0 },
     {"NumPartialOmits",0,  INT_TYPE,     &NumPartialOmits,     0,      0 },
     {"NumQueued",      0,  INT_TYPE,     &NumQueued,           0,      0 },
     {"NumTrig",        0,  INT_TYPE,     &NumTriggered,        0,      0 },
-    {"October",        1,  STR_TYPE,     &DynamicMonthName[9], 0,      0 },
-    {"On",             1,  STR_TYPE,     &DynamicOn,           0,      0 },
+    {"October",        1,  TRANS_TYPE,   "October",            0,      0 },
+    {"On",             1,  TRANS_TYPE,   "on",                 0,      0 },
     {"OnceFile",       1,  SPECIAL_TYPE, oncefile_func,        0,      0 },
     {"ParseUntriggered", 1, INT_TYPE,    &ParseUntriggered,    0,      1 },
-    {"Pm",             1,  STR_TYPE,     &DynamicPm,           0,      0 },
+    {"Pm",             1,  TRANS_TYPE,   "pm",                 0,      0 },
     {"PrefixLineNo",   0,  INT_TYPE,     &DoPrefixLineNo,      0,      0 },
     {"PSCal",          0,  INT_TYPE,     &PsCal,               0,      0 },
     {"RunOff",         0,  INT_TYPE,     &RunDisabled,         0,      0 },
-    {"Saturday",       1,  STR_TYPE,     &DynamicDayName[5],   0,      0 },
-    {"September",      1,  STR_TYPE,     &DynamicMonthName[8], 0,      0 },
+    {"Saturday",       1,  TRANS_TYPE,   "Saturday",           0,      0 },
+    {"September",      1,  TRANS_TYPE,   "September",          0,      0 },
     {"SimpleCal",      0,  INT_TYPE,     &DoSimpleCalendar,    0,      0 },
     {"SortByDate",     0,  INT_TYPE,     &SortByDate,          0,      0 },
     {"SortByPrio",     0,  INT_TYPE,     &SortByPrio,          0,      0 },
     {"SortByTime",     0,  INT_TYPE,     &SortByTime,          0,      0 },
     {"SubsIndent",     1,  INT_TYPE,     &SubsIndent,          0,      132 },
-    {"Sunday",         1,  STR_TYPE,     &DynamicDayName[6],   0,      0 },
+    {"Sunday",         1,  TRANS_TYPE,   "Sunday",             0,      0 },
+    {"SuppressImplicitWarnings", 1, INT_TYPE, &SuppressImplicitRemWarnings, 0, 1},
     {"SuppressLRM",    1,  INT_TYPE,     &SuppressLRM,         0,      1 },
     {"SysInclude",     0,  STR_TYPE,     &SysDir,              0,      0 },
     {"T",              0,  SPECIAL_TYPE, trig_date_func,       0,      0 },
     {"Td",             0,  SPECIAL_TYPE, trig_day_func,        0,      0 },
     {"TerminalBackground", 0, SPECIAL_TYPE,  terminal_bg_func, 0,      0 },
-    {"Thursday",       1,  STR_TYPE,     &DynamicDayName[3],   0,      0 },
+    {"Thursday",       1,  TRANS_TYPE,   "Thursday",           0,      0 },
     {"TimeSep",        1,  SPECIAL_TYPE, time_sep_func,        0,      0 },
     {"Tm",             0,  SPECIAL_TYPE, trig_mon_func,        0,      0 },
-    {"Today",          1,  STR_TYPE,     &DynamicToday,        0,      0 },
-    {"Tomorrow",       1,  STR_TYPE,     &DynamicTomorrow,     0,      0 },
+    {"Today",          1,  TRANS_TYPE,   "today",              0,      0 },
+    {"Tomorrow",       1,  TRANS_TYPE,   "tomorrow",           0,      0 },
     {"Tt",             0,  SPECIAL_TYPE, trig_time_func,       0,      0 },
-    {"Tuesday",        1,  STR_TYPE,     &DynamicDayName[1],   0,      0 },
+    {"Tuesday",        1,  TRANS_TYPE,   "Tuesday",            0,      0 },
     {"Tw",             0,  SPECIAL_TYPE, trig_wday_func,       0,      0 },
     {"Ty",             0,  SPECIAL_TYPE, trig_year_func,       0,      0 },
     {"U",              0,  SPECIAL_TYPE, today_date_func,      0,      0 },
@@ -950,17 +950,80 @@ static SysVar SysVarArr[] = {
     {"Um",             0,  SPECIAL_TYPE, today_mon_func,       0,      0 },
     {"UntimedFirst",   0,  INT_TYPE,     &UntimedBeforeTimed,  0,      0 },
     {"Use256Colors",   0,  INT_TYPE,     &Use256Colors,        0,      0 },
-    {"UseBGVTColors",  0,  INT_TYPE,     &UseBGVTColors,        0,      0 },
+    {"UseBGVTColors",  0,  INT_TYPE,     &UseBGVTColors,       0,      0 },
     {"UseTrueColors",  0,  INT_TYPE,     &UseTrueColors,       0,      0 },
     {"UseVTColors",    0,  INT_TYPE,     &UseVTColors,         0,      0 },
     {"Uw",             0,  SPECIAL_TYPE, today_wday_func,      0,      0 },
     {"Uy",             0,  SPECIAL_TYPE, today_year_func,      0,      0 },
-    {"Was",            1,  STR_TYPE,     &DynamicWas,          0,      0 },
-    {"Wednesday",      1,  STR_TYPE,     &DynamicDayName[2],   0,      0 }
+    {"Was",            1,  TRANS_TYPE,   "was",                0,      0 },
+    {"Wednesday",      1,  TRANS_TYPE,   "Wednesday",          0,      0 }
 };
 
 #define NUMSYSVARS ( sizeof(SysVarArr) / sizeof(SysVar) )
 static void DumpSysVar (char const *name, const SysVar *v);
+
+static int SetTranslatableVariable(SysVar *v, Value *value)
+{
+    return InsertTranslation((char const *) v->value, value->v.str);
+}
+
+static int GetTranslatableVariable(SysVar *v, Value *value)
+{
+    char const *translated = tr((char const *) v->value);
+    if (translated) {
+        value->v.str = StrDup(translated);
+    } else {
+        value->v.str = StrDup("");
+    }
+    if (!value->v.str) return E_NO_MEM;
+    value->type = STR_TYPE;
+    return OK;
+}
+
+static int SetSysVarHelper(SysVar *v, Value *value)
+{
+    int r;
+    if (!v->modifiable) {
+        Eprint("%s: `$%s'", GetErr(E_CANT_MODIFY), v->name);
+        return E_CANT_MODIFY;
+    }
+
+    if (v->type == TRANS_TYPE) {
+        if (value->type != STR_TYPE) return E_BAD_TYPE;
+        r =  SetTranslatableVariable(v, value);
+        DestroyValue(*value);
+        return r;
+    }
+
+    if (v->type != SPECIAL_TYPE &&
+        v->type != value->type) return E_BAD_TYPE;
+    if (v->type == SPECIAL_TYPE) {
+        SysVarFunc f = (SysVarFunc) v->value;
+        r = f(1, value);
+        DestroyValue(*value);
+        return r;
+    }
+
+    if (v->type == STR_TYPE) {
+        /* If it's already the same, don't bother doing anything */
+        if (!strcmp(value->v.str, * (char const **) v->value)) {
+            DestroyValue(*value);
+            return OK;
+        }
+
+        /* If it's a string variable, special measures must be taken */
+        if (v->been_malloced) free(*((char **)(v->value)));
+        v->been_malloced = 1;
+        *((char **) v->value) = value->v.str;
+        value->type = ERR_TYPE;  /* So that it's not accidentally freed */
+    } else {
+        if (v->max != ANY && value->v.val > v->max) return E_2HIGH;
+        if (v->min != ANY && value->v.val < v->min) return E_2LOW;
+        *((int *)v->value) = value->v.val;
+    }
+    return OK;
+}
+
 /***************************************************************/
 /*                                                             */
 /*  SetSysVar                                                  */
@@ -970,41 +1033,9 @@ static void DumpSysVar (char const *name, const SysVar *v);
 /***************************************************************/
 int SetSysVar(char const *name, Value *value)
 {
-    int r;
     SysVar *v = FindSysVar(name);
     if (!v) return E_NOSUCH_VAR;
-    if (!v->modifiable) {
-	Eprint("%s: `$%s'", ErrMsg[E_CANT_MODIFY], name);
-	return E_CANT_MODIFY;
-    }
-
-    if (v->type != SPECIAL_TYPE &&
-	v->type != value->type) return E_BAD_TYPE;
-    if (v->type == SPECIAL_TYPE) {
-	SysVarFunc f = (SysVarFunc) v->value;
-	r = f(1, value);
-        DestroyValue(*value);
-        return r;
-    }
-
-    if (v->type == STR_TYPE) {
-        /* If it's already the same, don't bother doing anything */
-        if (!strcmp(value->v.str, (char const *) v->value)) {
-            DestroyValue(*value);
-            return OK;
-        }
-
-        /* If it's a string variable, special measures must be taken */
-	if (v->been_malloced) free(*((char **)(v->value)));
-	v->been_malloced = 1;
-	*((char **) v->value) = value->v.str;
-	value->type = ERR_TYPE;  /* So that it's not accidentally freed */
-    } else {
-	if (v->max != ANY && value->v.val > v->max) return E_2HIGH;
-	if (v->min != ANY && value->v.val < v->min) return E_2LOW;
-	*((int *)v->value) = value->v.val;
-    }
-    return OK;
+    return SetSysVarHelper(v, value);
 }
 
 /***************************************************************/
@@ -1020,31 +1051,35 @@ int GetSysVar(char const *name, Value *val)
 
     val->type = ERR_TYPE;
     if (!v) return E_NOSUCH_VAR;
+    if (v->type == TRANS_TYPE) {
+        return GetTranslatableVariable(v, val);
+    }
+
     if (v->type == CONST_INT_TYPE) {
         val->v.val = v->constval;
         val->type = INT_TYPE;
         return OK;
     }
     if (v->type == SPECIAL_TYPE) {
-	SysVarFunc f = (SysVarFunc) v->value;
-	return f(0, val);
+        SysVarFunc f = (SysVarFunc) v->value;
+        return f(0, val);
     } else if (v->type == STR_TYPE) {
         if (! * (char **) v->value) {
             val->v.str = StrDup("");
         } else {
             val->v.str = StrDup(*((char **) v->value));
         }
-	if (!val->v.str) return E_NO_MEM;
+        if (!val->v.str) return E_NO_MEM;
     } else {
-	val->v.val = *((int *) v->value);
+        val->v.val = *((int *) v->value);
     }
     val->type = v->type;
 
     /* In "verbose" mode, print attempts to test $RunOff */
     if (DebugFlag & DB_PRTLINE) {
-	if (v->value == (void *) &RunDisabled) {
-	    Wprint("(Security note: $RunOff variable tested.)");
-	}
+        if (v->value == (void *) &RunDisabled) {
+            Wprint("(Security note: $RunOff variable tested.)");
+        }
     }
     return OK;
 }
@@ -1063,11 +1098,11 @@ SysVar *FindSysVar(char const *name)
     int r;
 
     while (top >= bottom) {
-	r = StrCmpi(name, SysVarArr[mid].name);
-	if (!r) return &SysVarArr[mid];
-	else if (r>0) bottom = mid+1;
-	else	    top = mid-1;
-	mid = (top+bottom) / 2;
+        r = StrCmpi(name, SysVarArr[mid].name);
+        if (!r) return &SysVarArr[mid];
+        else if (r>0) bottom = mid+1;
+        else        top = mid-1;
+        mid = (top+bottom) / 2;
     }
     return NULL;
 }
@@ -1086,8 +1121,8 @@ void DumpSysVarByName(char const *name)
     SysVar *v;
 
     if (!name || !*name) {
-	for (i=0; i<NUMSYSVARS; i++) DumpSysVar(name, SysVarArr + i);
-	return;
+        for (i=0; i<NUMSYSVARS; i++) DumpSysVar(name, SysVarArr + i);
+        return;
     }
 
     v = FindSysVar(name);
@@ -1112,40 +1147,49 @@ static void DumpSysVar(char const *name, const SysVar *v)
 
     buffer[0]='$'; buffer[1] = 0;
     if (name && strlen(name) > VAR_NAME_LEN) {
-	fprintf(ErrFp, "$%s: Name too long\n", name);
-	return;
+        fprintf(ErrFp, "$%s: Name too long\n", name);
+        return;
     }
     if (name) strcat(buffer, name); else strcat(buffer, v->name);
-    fprintf(ErrFp, "%16s  ", buffer);
+    fprintf(ErrFp, "%25s  ", buffer);
     if (v) {
         if (v->type == CONST_INT_TYPE) {
             fprintf(ErrFp, "%d\n", v->constval);
         } else if (v->type == SPECIAL_TYPE) {
-	    SysVarFunc f = (SysVarFunc) v->value;
-	    f(0, &vtmp);
-	    PrintValue(&vtmp, ErrFp);
-	    putc('\n', ErrFp);
-	    DestroyValue(vtmp);
-	} else if (v->type == STR_TYPE) {
+            SysVarFunc f = (SysVarFunc) v->value;
+            f(0, &vtmp);
+            PrintValue(&vtmp, ErrFp);
+            putc('\n', ErrFp);
+            DestroyValue(vtmp);
+        } else if (v->type == TRANS_TYPE) {
+            int r = GetSysVar(v->name, &vtmp);
+            if (r == OK) {
+                PrintValue(&vtmp, ErrFp);
+                putc('\n', ErrFp);
+                DestroyValue(vtmp);
+            } else {
+                fprintf(ErrFp, "Error: %s\n", GetErr(r));
+            }
+        } else if (v->type == STR_TYPE) {
             vtmp.type = STR_TYPE;
             vtmp.v.str = * ((char **)v->value);
             PrintValue(&vtmp, ErrFp);
-	    putc('\n', ErrFp);
-	} else if (v->type == DATE_TYPE) {
-	    vtmp.type = DATE_TYPE;
-	    vtmp.v.val = * (int *) v->value;
-	    PrintValue(&vtmp, ErrFp);
-	    putc('\n', ErrFp);
-	} else {
-	    if (!v->modifiable) fprintf(ErrFp, "%d\n", *((int *)v->value));
-	    else {
-		fprintf(ErrFp, "%-10d  ", *((int *)v->value));
-		if (v->min == ANY) fprintf(ErrFp, "(-Inf, ");
-		else                         fprintf(ErrFp, "[%d, ", v->min);
-		if (v->max == ANY) fprintf(ErrFp, "Inf)\n");
-		else                         fprintf(ErrFp, "%d]\n", v->max);
-	    }
-	}
+            putc('\n', ErrFp);
+        } else if (v->type == DATE_TYPE) {
+            vtmp.type = DATE_TYPE;
+            vtmp.v.val = * (int *) v->value;
+            PrintValue(&vtmp, ErrFp);
+            putc('\n', ErrFp);
+        } else {
+            if (!v->modifiable) fprintf(ErrFp, "%d\n", *((int *)v->value));
+            else {
+                fprintf(ErrFp, "%-10d  ", *((int *)v->value));
+                if (v->min == ANY) fprintf(ErrFp, "(-Inf, ");
+                else                         fprintf(ErrFp, "[%d, ", v->min);
+                if (v->max == ANY) fprintf(ErrFp, "Inf)\n");
+                else                         fprintf(ErrFp, "%d]\n", v->max);
+            }
+        }
     } else   fprintf(ErrFp, "%s\n", UNDEF);
 
     return;
@@ -1195,3 +1239,18 @@ set_components_from_lat_and_long(void)
     }
 }
 
+void
+print_sysvar_tokens(void)
+{
+    int i;
+    printf("\n# System Variables\n\n");
+    for (i=0; i< (int) NUMSYSVARS; i++) {
+        printf("$%s\n", SysVarArr[i].name);
+    }
+}
+
+void
+dump_var_hash_stats(void)
+{
+    hash_table_dump_stats(&VHashTbl, ErrFp);
+}
